@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: autoscan_inotify.cc 1698 2008-02-23 20:48:30Z lww $
+    $Id: autoscan_inotify.cc 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file autoscan_inotify.cc
@@ -128,6 +128,7 @@ void *AutoscanInotify::staticThreadProc(void *arg)
     log_debug("started inotify thread.\n");
     AutoscanInotify *inst = (AutoscanInotify *)arg;
     inst->threadProc();
+    Storage::getInstance()->threadCleanup();
     log_debug("exiting inotify thread...\n");
     pthread_exit(NULL);
     return NULL;
@@ -222,7 +223,7 @@ void AutoscanInotify::threadProc()
             {
                 int wd = event->wd;
                 int mask = event->mask;
-                String name(event->name);
+                String name = event->name;
                 log_debug("inotify event: %d %x %s\n", wd, mask, name.c_str());
                 
                 Ref<Wd> wdObj = watches->get(wd);
