@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: cds_objects.h 1698 2008-02-23 20:48:30Z lww $
+    $Id: cds_objects.h 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file cds_objects.h
@@ -43,6 +43,12 @@
 #define OBJECT_TYPE_ITEM_EXTERNAL_URL   0x00000008
 #define OBJECT_TYPE_ITEM_INTERNAL_URL   0x00000010
 
+#define STRING_OBJECT_TYPE_CONTAINER    "container"
+#define STRING_OBJECT_TYPE_ITEM         "item"
+#define STRING_OBJECT_TYPE_ACTIVE_ITEM  "active_item"
+#define STRING_OBJECT_TYPE_EXTERNAL_URL "external_url"
+#define STRING_OBJECT_TYPE_INTERNAL_URL "internal_url"
+
 #define IS_CDS_CONTAINER(type) (type & OBJECT_TYPE_CONTAINER)
 #define IS_CDS_ITEM(type) (type & OBJECT_TYPE_ITEM)
 #define IS_CDS_ACTIVE_ITEM(type) (type & OBJECT_TYPE_ACTIVE_ITEM)
@@ -58,8 +64,8 @@
 #define OBJECT_FLAG_PROXY_URL             0x00000020
 #define OBJECT_FLAG_ONLINE_SERVICE        0x00000040
 #define OBJECT_FLAG_OGG_THEORA            0x00000080
-//#define OBJECT_FLAG_XXX                 0x00000100
-//#define OBJECT_FLAG_XXX                 0x00000200
+#define OBJECT_FLAG_DVD_IMAGE             0x00000100
+#define OBJECT_FLAG_PLAYED                0x00000200
 
 #define OBJECT_AUTOSCAN_NONE    0
 #define OBJECT_AUTOSCAN_UI      1
@@ -241,6 +247,10 @@ public:
     /// \brief Query resources
     inline zmm::Ref<zmm::Array<CdsResource> > getResources()
     { return resources; }
+ 
+    /// \brief Set resources
+    inline void setResources(zmm::Ref<zmm::Array<CdsResource> > res) 
+    { resources = res; }
     
     /// \brief Query resource tag with the given index
     inline zmm::Ref<CdsResource> getResource(int index)
@@ -282,6 +292,9 @@ public:
     /// \brief Returns the path to the object as it appears in the database tree.
     virtual zmm::String getVirtualPath() = 0; 
     
+    static zmm::String mapObjectType(int objectType);
+    static int remapObjectType(zmm::String objectType);
+    
     friend int CdsObjectTitleComparator(void *arg1, void *arg2);
 };
 
