@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: cds_resource_manager.cc 1709 2008-03-01 15:35:51Z lww $
+    $Id: cds_resource_manager.cc 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file cds_resource_manager.cc
@@ -50,7 +50,8 @@ CdsResourceManager::CdsResourceManager() : Object()
 
 String CdsResourceManager::renderExtension(String contentType, String location)
 {
-    String ext = String(_URL_ARG_SEPARATOR) + _(URL_FILE_EXTENSION) + _("=");
+    String ext = _(_URL_PARAM_SEPARATOR) + URL_FILE_EXTENSION + 
+                _URL_PARAM_SEPARATOR + "file";
 
     if (string_ok(contentType) && (contentType != CONTENT_TYPE_PLAYLIST))
     {
@@ -67,7 +68,7 @@ String CdsResourceManager::renderExtension(String contentType, String location)
             // make sure that the extension does not contain the separator character
             if (string_ok(extension) && 
                (extension.index(URL_PARAM_SEPARATOR) == -1) &&
-               (extension.index(URL_ARG_SEPARATOR) == -1))
+               (extension.index(URL_PARAM_SEPARATOR) == -1))
             {
                 ext = ext + extension;
                 return ext;
@@ -85,15 +86,49 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
                     IS_CDS_ITEM_EXTERNAL_URL(item->getObjectType())) &&
                     (!item->getFlag(OBJECT_FLAG_PROXY_URL)));
 
+    bool isExtThumbnail = false; // this sucks
     Ref<Dictionary> mappings = config->getDictionaryOption(
                         CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
 
+#if defined(HAVE_FFMPEG) && defined(HAVE_FFMPEGTHUMBNAILER)
+    if (config->getBoolOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_ENABLED) && 
+       (item->getMimeType().startsWith(_("video")) || 
+        item->getFlag(OBJECT_FLAG_OGG_THEORA)))
+    {
+        String videoresolution = item->getResource(0)->getAttribute(MetadataHandler::getResAttrName(R_RESOLUTION));
+        int x;
+        int y;
+
+        if (string_ok(videoresolution) && 
+            check_resolution(videoresolution, &x, &y))
+        {
+            String thumb_mimetype = mappings->get(_(CONTENT_TYPE_JPG));
+            if (!string_ok(thumb_mimetype))
+                thumb_mimetype = _("image/jpeg");
+
+            Ref<CdsResource> ffres(new CdsResource(CH_FFTH));
+            ffres->addParameter(_(RESOURCE_HANDLER), String::from(CH_FFTH));
+            ffres->addAttribute(MetadataHandler::getResAttrName(R_PROTOCOLINFO),
+                    renderProtocolInfo(thumb_mimetype));
+            ffres->addOption(_(RESOURCE_CONTENT_TYPE), _(THUMBNAIL));
+
+            y = config->getIntOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE) * y / x;
+            x = config->getIntOption(CFG_SERVER_EXTOPTS_FFMPEGTHUMBNAILER_THUMBSIZE);
+            String resolution = String::from(x) + "x" + String::from(y);
+            ffres->addAttribute(MetadataHandler::getResAttrName(R_RESOLUTION),
+                    resolution);
+            item->addResource(ffres);
+            log_debug("Adding resource for video thumbnail\n");
+        }
+    }
+#endif // FFMPEGTHUMBNAILER
 
 #ifdef EXTERNAL_TRANSCODING
     // this will be used to count only the "real" resources, omitting the
     // transcoded ones
     int realCount = 0;
     bool hide_original_resource = false;
+    int original_resource = 0;
     
     Ref<UrlBase> urlBase_tr;
 
@@ -168,6 +203,9 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
                 }
             }
 
+            if (!item->getFlag(OBJECT_FLAG_DVD_IMAGE) && (tp->onlyDVD()))
+                continue;
+
             Ref<CdsResource> t_res(new CdsResource(CH_TRANSCODE));
             t_res->addParameter(_(URL_PARAM_TRANSCODE_PROFILE_NAME), tp->getName());
             // after transcoding resource was added we can not rely on
@@ -175,14 +213,57 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
             t_res->addOption(_(CONTENT_TYPE_OGG), 
                          item->getResource(0)->getOption(_(CONTENT_TYPE_OGG)));
             t_res->addParameter(_(URL_PARAM_TRANSCODE), _(URL_VALUE_TRANSCODE));
+
+            String targetMimeType = tp->getTargetMimeType();
+
+            if (!tp->isThumbnail())
+            {
+                // duration should be the same for transcoded media, so we can 
+                // take the value from the original resource
+                String duration = item->getResource(0)->getAttribute(MetadataHandler::getResAttrName(R_DURATION));
+                if (string_ok(duration))
+                    t_res->addAttribute(MetadataHandler::getResAttrName(R_DURATION),
+                            duration);
+
+                int freq = tp->getSampleFreq();
+                if (freq == SOURCE)
+                {
+                    String frequency = item->getResource(0)->getAttribute(MetadataHandler::getResAttrName(R_SAMPLEFREQUENCY));
+                    if (string_ok(frequency))
+                    {
+                        t_res->addAttribute(MetadataHandler::getResAttrName(R_SAMPLEFREQUENCY), frequency);
+                        targetMimeType = targetMimeType + _(";rate=") + 
+                                         frequency;
+                    }
+                }
+                else if (freq != OFF)
+                {
+                    t_res->addAttribute(MetadataHandler::getResAttrName(R_SAMPLEFREQUENCY), String::from(freq));
+                    targetMimeType = targetMimeType + _(";rate=") + 
+                                     String::from(freq);
+                }
+
+                int chan = tp->getNumChannels();
+                if (chan == SOURCE)
+                {
+                    String nchannels = item->getResource(0)->getAttribute(MetadataHandler::getResAttrName(R_NRAUDIOCHANNELS));
+                    if (string_ok(nchannels))
+                    {
+                        t_res->addAttribute(MetadataHandler::getResAttrName(R_NRAUDIOCHANNELS), nchannels);
+                        targetMimeType = targetMimeType + _(";channels=") + 
+                                         nchannels;
+                    }
+                }
+                else if (chan != OFF)
+                {
+                    t_res->addAttribute(MetadataHandler::getResAttrName(R_NRAUDIOCHANNELS), String::from(chan));
+                    targetMimeType = targetMimeType + _(";channels=") + 
+                                     String::from(chan);
+                }
+            }
+
             t_res->addAttribute(MetadataHandler::getResAttrName(R_PROTOCOLINFO),
-                    renderProtocolInfo(tp->getTargetMimeType()));
-            // duration should be the same for transcoded media, so we can take
-            // the value from the original resource
-            String duration = item->getResource(0)->getAttribute(MetadataHandler::getResAttrName(R_DURATION));
-            if (string_ok(duration))
-                t_res->addAttribute(MetadataHandler::getResAttrName(R_DURATION),
-                        duration);
+                    renderProtocolInfo(targetMimeType));
 
             if (tp->isThumbnail())
                 t_res->addOption(_(RESOURCE_CONTENT_TYPE), _(EXIF_THUMBNAIL));
@@ -193,7 +274,10 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
                 hide_original_resource = true;
 
             if (tp->firstResource())
+            {
                 item->insertResource(0, t_res);
+                original_resource++;
+            }
             else
                 item->addResource(t_res);
         }
@@ -202,7 +286,7 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
             urlBase_tr = addResources_getUrlBase(item, true);
     }
 
-#endif
+#endif // EXTERNAL_TRANSCODING
 
     int resCount = item->getResourceCount();
     for (int i = 0; i < resCount; i++)
@@ -230,7 +314,7 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
         // accounting for those dynamic resources: i.e. the parameter should
         // still only count the "real" resources, because that's what the
         // file request handler will be getting.
-        // the for transcoded resources the res_id can be safely ignored,
+        // for transcoded resources the res_id can be safely ignored,
         // because a transcoded resource is identified by the profile name
 #ifdef EXTERNAL_TRANSCODING
         // flag if we are dealing with the transcoded resource
@@ -264,16 +348,37 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
 #endif
         if ((res_params != nil) && (res_params->size() > 0))
         {
-            url = url + _(_URL_ARG_SEPARATOR);
-            url = url + res_params->encode();
+            url = url + _(_URL_PARAM_SEPARATOR);
+            url = url + res_params->encodeSimple();
+        }
+
+        // ok this really sucks, I guess another rewrite of the resource manager
+        // is necessary
+        if ((i > 0) && (item->getResource(i)->getHandlerType() == CH_EXTURL) &&
+           ((item->getResource(i)->getOption(_(RESOURCE_CONTENT_TYPE)) == 
+            THUMBNAIL) || 
+            (item->getResource(i)->getOption(_(RESOURCE_CONTENT_TYPE)) ==
+                            ID3_ALBUM_ART)))
+        {
+            url = item->getResource(i)->getOption(_(RESOURCE_OPTION_URL));
+            if (!string_ok(url))
+                throw _Exception(_("missing thumbnail URL!"));
+
+            isExtThumbnail = true;
         }
 
         /// \todo currently resource is misused for album art
 #ifdef HAVE_ID3_ALBUMART
         // only add upnp:AlbumArtURI if we have an AA, skip the resource
-        if ((i > 0) && (item->getResource(i)->getHandlerType() == CH_ID3))
+        if ((i > 0) && ((item->getResource(i)->getHandlerType() == CH_ID3) ||
+                        (item->getResource(i)->getHandlerType() == CH_MP4) ||
+                        (item->getResource(i)->getHandlerType() == CH_EXTURL)))
         {
-            String rct = item->getResource(i)->getParameter(_(RESOURCE_CONTENT_TYPE));
+            String rct;
+            if (item->getResource(i)->getHandlerType() == CH_EXTURL)
+                rct = item->getResource(i)->getOption(_(RESOURCE_CONTENT_TYPE));
+            else    
+                rct = item->getResource(i)->getParameter(_(RESOURCE_CONTENT_TYPE));
             if (rct == ID3_ALBUM_ART)
             {
                 Ref<Element> aa(new Element(MetadataHandler::getMetaFieldName(M_ALBUMARTURI)));
@@ -289,24 +394,24 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
                 }
 #endif
                 element->appendElementChild(aa);
+                continue;
             }
-            continue;
         }
 #endif
-
-#ifdef EXTERNAL_TRANSCODING
-
-        // when transcoding is enabled the first (zero) resource can be the
-        // transcoded stream, that means that we can only go with the
-        // content type here and that we will not limit ourselves to the
-        // first resource
-        if (!skipURL)
+        if (!isExtThumbnail)
         {
-            if (transcoded)
-                url = url + renderExtension(contentType, nil);
-            else
-                url = url + renderExtension(contentType, item->getLocation()); 
-        }
+#ifdef EXTERNAL_TRANSCODING
+            // when transcoding is enabled the first (zero) resource can be the
+            // transcoded stream, that means that we can only go with the
+            // content type here and that we will not limit ourselves to the
+            // first resource
+            if (!skipURL)
+            {
+                if (transcoded)
+                    url = url + renderExtension(contentType, nil);
+                else
+                    url = url + renderExtension(contentType, item->getLocation()); 
+            }
 #else
         // for non transcoded item we will only do this for the first
         // resource, that seemed to be sufficient so far (mostly this paramter
@@ -316,8 +421,9 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
         {
             url = url + renderExtension(contentType, item->getLocation());
         }
-#endif
 
+#endif
+        }
 #ifdef EXTEND_PROTOCOLINFO
         if (config->getBoolOption(CFG_SERVER_EXTEND_PROTOCOLINFO))
         {
@@ -340,7 +446,9 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
                (item->getResource(i)->getParameter(_(RESOURCE_CONTENT_TYPE)) 
                      == EXIF_THUMBNAIL)) || 
                (item->getResource(i)->getOption(_(RESOURCE_CONTENT_TYPE)) 
-                     == EXIF_THUMBNAIL)) &&
+                     == EXIF_THUMBNAIL) || 
+               (item->getResource(i)->getOption(_(RESOURCE_CONTENT_TYPE))
+                                              == THUMBNAIL)) &&
               (x <= 160) && (y <= 160))
                         extend = _(D_PROFILE) + "=" + D_JPEG_TN+";";
                     else if ((x <= 640) && (y <= 420))
@@ -355,9 +463,15 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
 #ifdef EXTERNAL_TRANSCODING
         // we do not support seeking at all, so 00
         // and the media is converted, so set CI to 1
-        if (transcoded)
+        if (!isExtThumbnail && transcoded)
+        {
             extend = extend + D_OP + "=00;" + 
                      D_CONVERSION_INDICATOR + "=" D_CONVERSION;
+
+            if (mimeType.startsWith(_("audio")) || 
+                mimeType.startsWith(_("video")))
+                extend = extend + ";" D_FLAGS "=" D_TR_FLAGS_AV;
+        }
         else
 #endif
         extend = extend + D_OP + "=01;" + 
@@ -372,7 +486,8 @@ void CdsResourceManager::addResources(Ref<CdsItem> item, Ref<Element> element)
         }
 #endif
 #ifdef EXTERNAL_TRANSCODING
-        if (!hide_original_resource || transcoded)
+        if (!hide_original_resource || transcoded || 
+           (hide_original_resource && (original_resource != i)))
 #endif
             element->appendElementChild(UpnpXML_DIDLRenderResource(url, res_attrs));
     }
@@ -394,9 +509,10 @@ Ref<CdsResourceManager::UrlBase> CdsResourceManager::addResources_getUrlBase(Ref
     int objectType = item->getObjectType();
     if (IS_CDS_ITEM_INTERNAL_URL(objectType))
     {
-        urlBase->urlBase = Server::getInstance()->getVirtualURL() + _("/") + 
+        urlBase->urlBase = Server::getInstance()->getVirtualURL() + 
+                           _(_URL_PARAM_SEPARATOR) + 
                            CONTENT_SERVE_HANDLER + 
-                           _("/") + item->getLocation();
+                           _(_URL_PARAM_SEPARATOR) + item->getLocation();
         return urlBase;
     }
 
@@ -411,19 +527,34 @@ Ref<CdsResourceManager::UrlBase> CdsResourceManager::addResources_getUrlBase(Ref
         if ((item->getFlag(OBJECT_FLAG_ONLINE_SERVICE) && 
                 item->getFlag(OBJECT_FLAG_PROXY_URL)) || forceLocal)
         {
-            urlBase->urlBase = Server::getInstance()->getVirtualURL() + _("/") +
+            urlBase->urlBase = Server::getInstance()->getVirtualURL() + 
+                _(_URL_PARAM_SEPARATOR) +
                 CONTENT_ONLINE_HANDLER + _(_URL_PARAM_SEPARATOR) +
-                dict->encode() + _(_URL_ARG_SEPARATOR) +
-                _(URL_RESOURCE_ID) + _("=");
+                dict->encodeSimple() + _(_URL_PARAM_SEPARATOR) +
+                _(URL_RESOURCE_ID) + _(_URL_PARAM_SEPARATOR);
             urlBase->addResID = true;
             return urlBase;
         }
     }
+
+#ifdef HAVE_LIBDVDNAV_DISABLED
+    if (IS_CDS_ITEM(objectType) && (item->getFlag(OBJECT_FLAG_DVD_IMAGE)))
+    {
+        urlBase->urlBase = Server::getInstance()->getVirtualURL() +
+                          _(_URL_PARAM_SEPARATOR) +
+                           CONTENT_DVD_IMAGE_HANDLER + _(_URL_PARAM_SEPARATOR) +
+                           dict->encodeSimple() + _(_URL_PARAM_SEPARATOR) +
+                           _(URL_RESOURCE_ID) + _(_URL_PARAM_SEPARATOR);
+        urlBase->addResID = true;
+        return urlBase;
+    }
+#endif
         
-    urlBase->urlBase = Server::getInstance()->getVirtualURL() + _("/") +
+    urlBase->urlBase = Server::getInstance()->getVirtualURL() +
+                       _(_URL_PARAM_SEPARATOR) +
                        CONTENT_MEDIA_HANDLER + _(_URL_PARAM_SEPARATOR) + 
-                       dict->encode() + _(_URL_ARG_SEPARATOR) + 
-                       _(URL_RESOURCE_ID) + _("=");
+                       dict->encodeSimple() + _(_URL_PARAM_SEPARATOR) + 
+                       _(URL_RESOURCE_ID) + _(_URL_PARAM_SEPARATOR);
     urlBase->addResID = true;
     return urlBase;
 }
@@ -437,4 +568,3 @@ String CdsResourceManager::getFirstResource(Ref<CdsItem> item)
     else
         return urlBase->urlBase;
 }
-
