@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: file_request_handler.cc 1698 2008-02-23 20:48:30Z lww $
+    $Id: file_request_handler.cc 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file file_request_handler.cc
@@ -51,6 +51,17 @@
 #include "file_request_handler.h"
 #include "metadata_handler.h"
 #include "tools.h"
+#include "play_hook.h"
+
+#ifdef HAVE_LIBDVDNAV
+    #include "dvd_io_handler.h"
+    #include "fd_io_handler.h"
+    #include "metadata/dvd_handler.h"
+    #include "mpegremux_processor.h"
+    #include "thread_executor.h"
+    #include "io_handler_chainer.h"
+#endif
+
 #ifdef EXTERNAL_TRANSCODING
     #include "transcoding/transcode_dispatcher.h"
 #endif
@@ -76,15 +87,14 @@ void FileRequestHandler::get_info(IN const char *filename, OUT struct File_Info
     int ret = 0;
     bool is_srt = false;
 
-    String url_path, parameters;
-    split_url(filename, URL_PARAM_SEPARATOR, url_path, parameters);
-    
+    String parameters = (filename + strlen(LINK_FILE_REQUEST_HANDLER));
+
     Ref<Dictionary> dict(new Dictionary());
-    dict->decode(parameters);
+    dict->decodeSimple(parameters);
+
+    log_debug("full url (filename): %s, parameters: %s\n",
+            filename, parameters.c_str());
 
-    log_debug("full url (filename): %s, url_path: %s, parameters: %s\n",
-               filename, url_path.c_str(), parameters.c_str());
-    
     String objID = dict->get(_("object_id"));
     if (objID == nil)
     {
@@ -104,28 +114,31 @@ void FileRequestHandler::get_info(IN const char *filename, OUT struct File_Info
 
     if (!IS_CDS_ITEM(objectType))
     {
-        throw _Exception(_("get_info: object is not an item"));
+        throw _Exception(_("requested object is not an item"));
     }
-     
+
     Ref<CdsItem> item = RefCast(obj, CdsItem);
 
     String path = item->getLocation();
-    
+
     // determining which resource to serve 
     int res_id = 0;
     String s_res_id = dict->get(_(URL_RESOURCE_ID));
 #ifdef EXTERNAL_TRANSCODING
     if (string_ok(s_res_id) && (s_res_id != _(URL_VALUE_TRANSCODE_NO_RES_ID)))
 #else
-    if (string_ok(s_res_id))
+        if (string_ok(s_res_id))
 #endif
-        res_id = s_res_id.toInt();
-    else
-        res_id = -1;
-
+            res_id = s_res_id.toInt();
+        else
+            res_id = -1;
 
     String ext = dict->get(_("ext"));
-    if (ext == ".srt")
+    int edot = ext.rindex('.');
+    if (edot > -1)
+        ext = ext.substring(edot);
+    if ((ext == ".srt") || (ext == ".ssa") ||
+            (ext == ".smi") || (ext == ".sub"))
     {
         int dot = path.rindex('.');
         if (dot > -1)
@@ -140,7 +153,7 @@ void FileRequestHandler::get_info(IN const char *filename, OUT struct File_Info
         res_id = 0;
         is_srt = true;
     }
-   
+
     ret = stat(path.c_str(), &statbuf);
     if (ret != 0)
     {
@@ -172,36 +185,48 @@ void FileRequestHandler::get_info(IN const char *filename, OUT struct File_Info
 
 
             header = _("Content-Disposition: attachment; filename=\"") + 
-                     path.substring(slash_pos) + _("\"");
+                path.substring(slash_pos) + _("\"");
         }
     }
- 
+
 #ifdef EXTERNAL_TRANSCODING
     tr_profile = dict->get(_(URL_PARAM_TRANSCODE_PROFILE_NAME));
 #endif
- 
+
     info->http_header = NULL;
     // for transcoded resourecs res_id will always be negative
     log_debug("fetching resource id %d\n", res_id);
-    if ((res_id > 0) && (res_id < item->getResourceCount()))
+    String rh = dict->get(_(RESOURCE_HANDLER));
+
+    if (((res_id > 0) && (res_id < item->getResourceCount())) ||
+            ((res_id > 0) && string_ok(rh)))
     {
-        // http-get:*:image/jpeg:*
-        String protocolInfo = item->getResource(res_id)->getAttributes()->get(_("protocolInfo"));
-        if (protocolInfo != nil)
-        {
-            mimeType = getMTFromProtocolInfo(protocolInfo);
-        }
 
-        if (!string_ok(mimeType))
-            mimeType = _(MIMETYPE_DEFAULT);
-      
         log_debug("setting content length to unknown\n");
         /// \todo we could figure out the content length...
         info->file_length = -1;
-        Ref<CdsResource> resource = item->getResource(res_id);
-        Ref<MetadataHandler> h = MetadataHandler::createHandler(resource->getHandlerType());
-/*        Ref<IOHandler> io_handler = */ h->serveContent(item, res_id, &(info->file_length));
-        
+
+        int res_handler;
+        if (string_ok(rh))
+            res_handler = rh.toInt();
+        else
+        {
+            Ref<CdsResource> resource = item->getResource(res_id);
+            res_handler = resource->getHandlerType();
+            // http-get:*:image/jpeg:*
+            String protocolInfo = item->getResource(res_id)->getAttributes()->get(_("protocolInfo"));
+            if (protocolInfo != nil)
+            {
+                mimeType = getMTFromProtocolInfo(protocolInfo);
+            }
+        }
+
+        Ref<MetadataHandler> h = MetadataHandler::createHandler(res_handler);
+        if (!string_ok(mimeType))
+            mimeType = h->getMimeType();
+
+        /*        Ref<IOHandler> io_handler = */ h->serveContent(item, res_id, &(info->file_length));
+
     }
     else
     {
@@ -213,47 +238,96 @@ void FileRequestHandler::get_info(IN const char *filename, OUT struct File_Info
 
             if (tp == nil)
                 throw _Exception(_("Transcoding of file ") + path +
-                                 " but no profile matching the name " +
-                                 tr_profile + " found");
+                        " but no profile matching the name " +
+                        tr_profile + " found");
 
             mimeType = tp->getTargetMimeType();
+            
+            Ref<Dictionary> mappings = ConfigManager::getInstance()->getDictionaryOption(CFG_IMPORT_MAPPINGS_MIMETYPE_TO_CONTENTTYPE_LIST);
+            if (mappings->get(mimeType) == CONTENT_TYPE_PCM)
+            {
+                String freq = item->getResource(0)->getAttribute(MetadataHandler::getResAttrName(R_SAMPLEFREQUENCY));
+                String nrch = item->getResource(0)->getAttribute(MetadataHandler::getResAttrName(R_NRAUDIOCHANNELS));
+
+                if (string_ok(freq))
+                    mimeType = mimeType + _(";rate=") + freq;
+                if (string_ok(nrch))
+                    mimeType = mimeType + _(";channels=") + nrch;
+            }
+
             info->file_length = -1;
         }
         else
 #endif
-        {
-            info->file_length = statbuf.st_size;
-            // if we are dealing with a regular file we should add the
-            // Accept-Ranges: bytes header, in order to indicate that we support
-            // seeking
-            if (S_ISREG(statbuf.st_mode))
+#ifdef HAVE_LIBDVDNAV
+            if (!is_srt && item->getFlag(OBJECT_FLAG_DVD_IMAGE))
             {
-                if (string_ok(header))
-                    header = header + _("\r\n");
+                String tmp = dict->get(DVDHandler::renderKey(DVD_Title));
+                if (!string_ok(tmp))
+                    throw _Exception(_("DVD Image requested but title parameter is missing!"));
+                int title = tmp.toInt();
+                if (title < 0)
+                    throw _Exception(_("DVD Image - requested invalid title!"));
 
-                header = header + _("Accept-Ranges: bytes");
-                /// \todo turned out that we are not always allowed to add this
-                /// header, since chunked encoding may be active and we do not
-                /// know that here
+                tmp = dict->get(DVDHandler::renderKey(DVD_Chapter));
+                if (!string_ok(tmp))
+                    throw _Exception(_("DVD Image requested but chapter parameter is missing!"));
+                int chapter = tmp.toInt();
+                if (chapter < 0)
+                    throw _Exception(_("DVD Image - requested invalid chapter!"));
+
+                // actually we are retrieving the stream id here
+                tmp = dict->get(DVDHandler::renderKey(DVD_AudioStreamID));
+                if (!string_ok(tmp))
+                    throw _Exception(_("DVD Image requested but audio track parameter is missing!"));
+                int audio_track = tmp.toInt();
+                if (audio_track < 0)
+                    throw _Exception(_("DVD Image - requested invalid audio stream ID!"));
+
+                /// \todo make sure we can seek in the streams
+                info->file_length = -1;
+                header = nil;
+            }
+            else
+#endif
+            {
+                info->file_length = statbuf.st_size;
+                // if we are dealing with a regular file we should add the
+                // Accept-Ranges: bytes header, in order to indicate that we support
+                // seeking
+                if (S_ISREG(statbuf.st_mode))
+                {
+                    if (string_ok(header))
+                        header = header + _("\r\n");
+
+                    header = header + _("Accept-Ranges: bytes");
+                    /// \todo turned out that we are not always allowed to add this
+                    /// header, since chunked encoding may be active and we do not
+                    /// know that here
+                }
             }
-        }
 
         if (!string_ok(mimeType))
             mimeType = item->getMimeType();
 
         //log_debug("sizeof off_t %d, statbuf.st_size %d\n", sizeof(off_t), sizeof(statbuf.st_size));
         //log_debug("get_info: file_length: " OFF_T_SPRINTF "\n", statbuf.st_size);
-        if (string_ok(header))
-            info->http_header = ixmlCloneDOMString(header.c_str());
     }
-        
+   
+#ifdef EXTEND_PROTOCOLINFO
+    header = getDLNAtransferHeader(mimeType, header);
+#endif
+
+    if (string_ok(header))
+        info->http_header = ixmlCloneDOMString(header.c_str());
+
     info->last_modified = statbuf.st_mtime;
     info->is_directory = S_ISDIR(statbuf.st_mode);
-   
+
     info->content_type = ixmlCloneDOMString(mimeType.c_str());
 
-   //    log_debug("get_info: Requested %s, ObjectID: %s, Location: %s\n, MimeType: %s\n",
-//          filename, object_id.c_str(), path.c_str(), info->content_type);
+    //    log_debug("get_info: Requested %s, ObjectID: %s, Location: %s\n, MimeType: %s\n",
+    //          filename, object_id.c_str(), path.c_str(), info->content_type);
 
     log_debug("web_get_info(): end\n");
 }
@@ -277,12 +351,13 @@ Ref<IOHandler> FileRequestHandler::open(IN const char *filename, OUT struct File
         throw _Exception(_("UPNP_WRITE unsupported"));
 
     String url_path, parameters;
-    split_url(filename, URL_PARAM_SEPARATOR, url_path, parameters);
+
+    parameters = (filename + strlen(LINK_FILE_REQUEST_HANDLER));
 
     Ref<Dictionary> dict(new Dictionary());
-    dict->decode(parameters);
-    log_debug("full url (filename): %s, url_path: %s, parameters: %s\n",
-               filename, url_path.c_str(), parameters.c_str());
+    dict->decodeSimple(parameters);
+    log_debug("full url (filename): %s, parameters: %s\n",
+            filename, parameters.c_str());
 
     String objID = dict->get(_("object_id"));
     if (objID == nil)
@@ -301,11 +376,24 @@ Ref<IOHandler> FileRequestHandler::open(IN const char *filename, OUT struct File
 
     if (!IS_CDS_ITEM(objectType))
     {
-        throw _Exception(_("object is not an item"));
+        throw _Exception(_("requested object is not an item"));
     }
 
+    // determining which resource to serve 
+    int res_id = 0;
+    String s_res_id = dict->get(_(URL_RESOURCE_ID));
+#ifdef EXTERNAL_TRANSCODING
+    if (string_ok(s_res_id) && (s_res_id != _(URL_VALUE_TRANSCODE_NO_RES_ID)))
+#else
+        if (string_ok(s_res_id))
+#endif
+            res_id = s_res_id.toInt();
+        else
+            res_id = -1;
+
+
     // update item info by running action
-    if (IS_CDS_ACTIVE_ITEM(objectType)) // check - if thumbnails, then no action, just show
+    if (IS_CDS_ACTIVE_ITEM(objectType) && (res_id == 0)) // check - if thumbnails, then no action, just show
     {
         Ref<CdsActiveItem> aitem = RefCast(obj, CdsActiveItem);
 
@@ -320,12 +408,12 @@ Ref<IOHandler> FileRequestHandler::open(IN const char *filename, OUT struct File
         log_debug("Script input: %s\n", input.c_str());
         if(strncmp(action.c_str(), "http://", 7))
         {
-#ifdef LOG_TOMBDEBUG
+#ifdef TOMBDEBUG
             struct timespec before;
             getTimespecNow(&before);
 #endif
             output = run_simple_process(action, _("run"), input);
-#ifdef LOG_TOMBDEBUG
+#ifdef TOMBDEBUG
             long delta = getDeltaMillis(&before);
             log_debug("script executed in %ld milliseconds\n", delta);
 #endif
@@ -371,20 +459,12 @@ Ref<IOHandler> FileRequestHandler::open(IN const char *filename, OUT struct File
 
     String path = item->getLocation();
 
-    // determining which resource to serve 
-    int res_id = 0;
-    String s_res_id = dict->get(_(URL_RESOURCE_ID));
-#ifdef EXTERNAL_TRANSCODING
-    if (string_ok(s_res_id) && (s_res_id != _(URL_VALUE_TRANSCODE_NO_RES_ID)))
-#else
-    if (string_ok(s_res_id))
-#endif
-        res_id = s_res_id.toInt();
-    else
-        res_id = -1;
-
     String ext = dict->get(_("ext"));
-    if (ext == ".srt")
+    int edot = ext.rindex('.');
+    if (edot > -1)
+        ext = ext.substring(edot);
+    if ((ext == ".srt") || (ext == ".ssa") ||
+            (ext == ".smi") || (ext == ".sub"))
     {
         int dot = path.rindex('.');
         if (dot > -1)
@@ -433,7 +513,7 @@ Ref<IOHandler> FileRequestHandler::open(IN const char *filename, OUT struct File
 
 
             header = _("Content-Disposition: attachment; filename=\"") + 
-                     path.substring(slash_pos) + _("\"");
+                path.substring(slash_pos) + _("\"");
         }
     }
     log_debug("fetching resource id %d\n", res_id);
@@ -454,23 +534,44 @@ Ref<IOHandler> FileRequestHandler::open(IN const char *filename, OUT struct File
     info->http_header = NULL;
     // Per default and in case of a bad resource ID, serve the file
     // itself
-    if ((res_id > 0) && (res_id < item->getResourceCount()))
+
+    // some resources are created dynamically and not saved in the database,
+    // so we can not load such a resource for a particular item, we will have
+    // to trust the resource handler parameter
+    String rh = dict->get(_(RESOURCE_HANDLER));
+    if (((res_id > 0) && (res_id < item->getResourceCount())) ||
+        ((res_id > 0) && string_ok(rh)))
     {
-        // http-get:*:image/jpeg:*
-        String protocolInfo = item->getResource(res_id)->getAttributes()->get(_("protocolInfo"));
-        if (protocolInfo != nil)
+        info->file_length = -1;
+
+        int res_handler;
+        if (string_ok(rh))
+            res_handler = rh.toInt();
+        else
         {
-            mimeType = getMTFromProtocolInfo(protocolInfo);
+            Ref<CdsResource> resource = item->getResource(res_id);
+            res_handler = resource->getHandlerType();
+            // http-get:*:image/jpeg:*
+            String protocolInfo = item->getResource(res_id)->getAttributes()->get(_("protocolInfo"));
+            if (protocolInfo != nil)
+            {
+                mimeType = getMTFromProtocolInfo(protocolInfo);
+            }
         }
 
+        Ref<MetadataHandler> h = MetadataHandler::createHandler(res_handler);
+        
         if (!string_ok(mimeType))
-            mimeType = _(MIMETYPE_DEFAULT);
+            mimeType = h->getMimeType();
 
+#ifdef EXTEND_PROTOCOLINFO
+        header = getDLNAtransferHeader(mimeType, header);
+#endif
+
+        if (string_ok(header))
+                info->http_header = ixmlCloneDOMString(header.c_str());
 
         info->content_type = ixmlCloneDOMString(mimeType.c_str());
-        info->file_length = -1;
-        Ref<CdsResource> resource = item->getResource(res_id);
-        Ref<MetadataHandler> h = MetadataHandler::createHandler(resource->getHandlerType());
         Ref<IOHandler> io_handler = h->serveContent(item, res_id, &(info->file_length));
         io_handler->open(mode);
         return io_handler;
@@ -482,10 +583,83 @@ Ref<IOHandler> FileRequestHandler::open(IN const char *filename, OUT struct File
         {
             Ref<TranscodeDispatcher> tr_d(new TranscodeDispatcher());
             Ref<TranscodingProfile> tp = ConfigManager::getInstance()->getTranscodingProfileListOption(CFG_TRANSCODING_PROFILE_LIST)->getByName(tr_profile);
-            return tr_d->open(tp, path, item->getObjectType(), info);
+            return tr_d->open(tp, path, RefCast(item, CdsObject), info);
         }
         else
 #endif
+#ifdef HAVE_LIBDVDNAV
+        if (!is_srt && item->getFlag(OBJECT_FLAG_DVD_IMAGE))
+        {
+            String tmp = dict->get(DVDHandler::renderKey(DVD_Title));
+            if (!string_ok(tmp))
+                throw _Exception(_("DVD Image requested but title parameter is missing!"));
+            int title = tmp.toInt();
+            if (title < 0)
+                throw _Exception(_("DVD Image - requested invalid title!"));
+
+            tmp = dict->get(DVDHandler::renderKey(DVD_Chapter));
+            if (!string_ok(tmp))
+                throw _Exception(_("DVD Image requested but chapter parameter is missing!"));
+            int chapter = tmp.toInt();
+            if (chapter < 0)
+                throw _Exception(_("DVD Image - requested invalid chapter!"));
+
+            // actually we are retrieving the audio stream id here
+            tmp = dict->get(DVDHandler::renderKey(DVD_AudioStreamID));
+            if (!string_ok(tmp))
+                throw _Exception(_("DVD Image requested but audio track parameter is missing!"));
+            int audio_track = tmp.toInt();
+            if (audio_track < 0)
+                throw _Exception(_("DVD Image - requested invalid audio stream ID!"));
+
+            /// \todo make sure we can seek in the streams
+            info->file_length = -1;
+            info->force_chunked = 1;
+            header = nil;
+            if (mimeType == nil)
+                mimeType = item->getMimeType();
+
+            info->content_type = ixmlCloneDOMString(mimeType.c_str());
+            log_debug("Serving dvd image %s Title: %d Chapter: %d\n",
+                    path.c_str(), title, chapter);
+            /// \todo add angle support
+            Ref<IOHandler> dvd_io_handler(new DVDIOHandler(path, title, chapter,
+                           audio_track));
+
+            int from_dvd_fd[2];
+            if (pipe(from_dvd_fd) == -1)
+                throw _Exception(_("Failed to create DVD input pipe!"));
+
+            int from_remux_fd[2];
+            if (pipe(from_remux_fd) == -1)
+            {
+                close(from_dvd_fd[0]);
+                close(from_dvd_fd[1]);
+                throw _Exception(_("Failed to create remux output pipe!"));
+            }
+
+            Ref<IOHandler> fd_writer(new FDIOHandler(from_dvd_fd[1]));
+            Ref<ThreadExecutor> from_dvd(new IOHandlerChainer(dvd_io_handler, 
+                                                        fd_writer, 16384));
+
+            Ref<IOHandler> fd_reader(new FDIOHandler(from_remux_fd[0]));
+            fd_reader->open(mode);
+            
+            Ref<MPEGRemuxProcessor> remux(new MPEGRemuxProcessor(from_dvd_fd[0],
+                                          from_remux_fd[1], 
+                                          (unsigned char)audio_track));
+
+            RefCast(fd_reader, FDIOHandler)->addReference(RefCast(remux, Object));
+            RefCast(fd_reader, FDIOHandler)->addReference(RefCast(from_dvd, Object));
+            RefCast(fd_reader, FDIOHandler)->addReference(RefCast(fd_writer, Object));
+            RefCast(fd_reader, FDIOHandler)->closeOther(fd_writer);
+
+            PlayHook::getInstance()->trigger(obj);
+            return fd_reader;
+        }
+        else
+#endif
+
         {
             if (mimeType == nil)
                 mimeType = item->getMimeType();
@@ -506,12 +680,16 @@ Ref<IOHandler> FileRequestHandler::open(IN const char *filename, OUT struct File
                 header = header + _("Accept-Ranges: bytes");
             }
 
+#ifdef EXTEND_PROTOCOLINFO
+            header = getDLNAtransferHeader(mimeType, header);
+#endif
             if (string_ok(header))
                 info->http_header = ixmlCloneDOMString(header.c_str());
 
 
             Ref<IOHandler> io_handler(new FileIOHandler(path));
             io_handler->open(mode);
+            PlayHook::getInstance()->trigger(obj);
             return io_handler;
         }
     }
