@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: layout.h 1698 2008-02-23 20:48:30Z lww $
+    $Id: layout.h 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file layout.h
@@ -37,7 +37,7 @@
 class Layout : public zmm::Object
 {
 public:
-    virtual void processCdsObject(zmm::Ref<CdsObject> obj) = 0;
+    virtual void processCdsObject(zmm::Ref<CdsObject> obj, zmm::String rootpath) = 0;
 };
 
 #endif // __LAYOUT_H__
