@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: main.cc 1699 2008-02-23 21:01:32Z lww $
+    $Id: main.cc 2086 2010-03-25 15:46:04Z jin_eld $
 */
 
 /// \file main.cc
@@ -68,7 +68,7 @@
 #include <limits.h>
 
 #ifdef HAVE_GETOPT_LONG
-    #define OPTSTR "i:e:p:c:m:f:u:g:a:l:P:dh"
+    #define OPTSTR "i:e:p:c:m:f:u:g:a:l:P:dhD"
 #endif
 
 using namespace zmm;
@@ -78,6 +78,14 @@ int restart_flag = 0;
 pthread_t main_thread_id;
 Ref<Timer> timer = nil;
 
+void print_copyright()
+{
+    printf("\nMediaTomb UPnP Server version %s - %s\n\n", VERSION, 
+            DESC_MANUFACTURER_URL);
+    printf("===============================================================================\n");
+    printf("Copyright 2005-2010 Gena Batsyan, Sergey Bostandzhyan, Leonhard Wimmer.\n");
+    printf("MediaTomb is free software, covered by the GNU General Public License version 2\n\n");
+}
 void signal_handler(int signum);
 
 int main(int argc, char **argv, char **envp)
@@ -88,27 +96,32 @@ int main(int argc, char **argv, char **envp)
     struct   sigaction action;
     sigset_t mask_set;
 
-    int      devnull;
+    int      devnull = -1;
+    int      redirect1 = -1;
+    int      redirect2 = -1;
     struct   passwd *pwd;
     struct   group  *grp;
 #ifdef HAVE_GETOPT_LONG
     int      opt_index = 0;
     int      o;
     static struct option long_options[] = {
-        {"ip", 1, 0, 'i'},
-        {"interface", 1, 0, 'e'},
-        {"port", 1, 0, 'p'},
-        {"config", 1, 0, 'c'},
-        {"home", 1, 0, 'm'},
-        {"cfgdir", 1, 0, 'f'},
-        {"user", 1, 0, 'u'},
-        {"group", 1, 0, 'g'},
-        {"daemon", 0, 0, 'd'},
-        {"pidfile", 1, 0, 'P'},
-        {"add", 1, 0, 'a'},
-        {"logfile", 1, 0, 'l'},
-        {"help", 0, 0, 'h'},
-        {0, 0, 0, 0}
+        {"ip", 1, 0, 'i'},          // 0
+        {"interface", 1, 0, 'e'},   // 1
+        {"port", 1, 0, 'p'},        // 2
+        {"config", 1, 0, 'c'},      // 3
+        {"home", 1, 0, 'm'},        // 4
+        {"cfgdir", 1, 0, 'f'},      // 5
+        {"user", 1, 0, 'u'},        // 6
+        {"group", 1, 0, 'g'},       // 7
+        {"daemon", 0, 0, 'd'},      // 8
+        {"pidfile", 1, 0, 'P'},     // 9
+        {"add", 1, 0, 'a'},         // 10
+        {"logfile", 1, 0, 'l'},     // 11
+        {"debug", 0, 0, 'D'},       // 12
+        {"compile-info", 0, 0, 0},  // 13
+        {"version", 0, 0, 0},       // 14
+        {"help", 0, 0, 'h'},        // 15
+        {0, 0, 0, 0}                
     };
 #endif
 
@@ -118,10 +131,13 @@ int main(int argc, char **argv, char **envp)
     String user;
     String group;
     String pid_file;
+    FILE* pid_fd = NULL;
     String interface;
     String ip;
     String prefix;
     String magic;
+    bool debug_logging = false;
+    bool print_version = false;
 
     Ref<Array<StringBase> > addFile(new Array<StringBase>());
 
@@ -131,7 +147,7 @@ int main(int argc, char **argv, char **envp)
     if (preload != NULL)
         ld_preload = String(preload);
 
-    if ((preload == NULL) || (ld_preload.find("0@0") != -1))
+    if ((preload == NULL) || (ld_preload.find("0@0") == -1))
     {
         printf("MediaTomb: Solaris check failed!\n");
         printf("Please set the environment to match glibc behaviour!\n");
@@ -150,12 +166,12 @@ int main(int argc, char **argv, char **envp)
         {
             case 'i':
                 log_debug("Option ip with param %s\n", optarg);
-                ip = String(optarg);
+                ip = optarg;
                 break;
 
             case 'e':
                 log_debug("Option interface with param %s\n", optarg);
-                interface = String(optarg);
+                interface = optarg;
                 break;
 
             case 'p':
@@ -177,7 +193,7 @@ int main(int argc, char **argv, char **envp)
 
             case 'c':
                 log_debug("Option config with param %s\n", optarg);
-                config_file = String(optarg);
+                config_file = optarg;
                 break;
 
             case 'd':
@@ -187,22 +203,22 @@ int main(int argc, char **argv, char **envp)
 
             case 'u':
                 log_debug("Running as user: %s\n", optarg);
-                user = String(optarg);
+                user = optarg;
                 break;
 
             case 'g':
                 log_debug("Running as group: %s\n", optarg);
-                group = String(optarg);
+                group = optarg;
                 break;
                 
             case 'a':
                 log_debug("Adding file/directory:: %s\n", optarg);
-                addFile->append(String(optarg));
+                addFile->append(String::copy(optarg));
                 break;
 
             case 'P':
                 log_debug("Pid file: %s\n", optarg);
-                pid_file = String(optarg);
+                pid_file = optarg;
                 break;
                 
             case 'l':
@@ -212,21 +228,33 @@ int main(int argc, char **argv, char **envp)
 
             case 'm':
                 log_debug("Home setting: %s\n", optarg);
-                home = String(optarg);
+                home = optarg;
                 break;
               
             case 'f':
                 log_debug("Confdir setting: %s\n", optarg);
-                confdir = String(optarg);
+                confdir = optarg;
+                break;
+                
+            case 'D':
+                
+#ifndef DEBUG_LOG_ENABLED
+                print_copyright();
+                printf("ERROR: MediaTomb wasn't compiled with debug output, but was run with -D/--debug.\n");
+                exit(EXIT_FAILURE);
+#endif
+                log_debug("enabling debug output\n");
+                debug_logging = true;
                 break;
             case '?':
             case 'h':
+                print_copyright();
                 printf("Usage: mediatomb [options]\n\
                         \n\
 Supported options:\n\
     --ip or -i         ip address to bind to\n\
     --interface or -e  network interface to bind to\n\
-    --port or -p       server port (the SDK only permits values => 49152)\n\
+    --port or -p       server port (the SDK only permits values >= 49152)\n\
     --config or -c     configuration file to use\n\
     --daemon or -d     run server in background\n\
     --home or -m       define the home directory\n\
@@ -236,12 +264,30 @@ Supported options:\n\
     --group or -g      run server under specified group\n\
     --add or -a        add the given file/directory\n\
     --logfile or -l    log to specified file\n\
+    --debug or -D      enable debug output\n\
+    --compile-info     print configuration summary and exit\n\
+    --version          print version information and exit\n\
     --help or -h       this help message\n\
 \n\
 For more information visit " DESC_MANUFACTURER_URL "\n\n");
 
                 exit(EXIT_FAILURE);
-
+                break;
+            case 0:
+                switch (opt_index)
+                {
+                case 13: // --compile-info
+                    print_copyright();
+                    printf("Compile info:\n");
+                    printf("-------------\n");
+                    printf("%s\n\n", COMPILE_INFO);
+                    exit(EXIT_SUCCESS);
+                    break;
+                case 14: // --version
+                    print_version = true;
+                    break;
+                }
+                break;
             default:
                 break;
          }
@@ -250,13 +296,23 @@ For more information visit " DESC_MANUFACTURER_URL "\n\n");
     log_warning("No getopt_long() support, all command line options disabled");
 #endif
 
-    if (!daemon)
+    if (print_version || ! daemon)
     {
-        printf("\nMediaTomb UPnP Server version %s - %s\n\n", VERSION, 
-                DESC_MANUFACTURER_URL);
-        printf("===============================================================================\n");
-        printf("Copyright 2005-2008 Gena Batsyan, Sergey Bostandzhyan, Leonhard Wimmer.\n");
-        printf("MediaTomb is free software, covered by the GNU General Public License version 2\n\n");
+        print_copyright();
+       
+        if (print_version)
+            exit(EXIT_FAILURE);
+    }
+
+   // create pid file before dropping privileges
+    if (pid_file != nil)
+    {
+        pid_fd = fopen(pid_file.c_str(), "w");
+        if (pid_fd == NULL)
+        {
+            log_error("Could not write pid file %s : %s\n",
+                       pid_file.c_str(), strerror(errno));
+        }
     }
 
     // check if user and/or group parameter was specified and try to run the server
@@ -316,13 +372,13 @@ For more information visit " DESC_MANUFACTURER_URL "\n\n");
 #ifndef __CYGWIN__
             char *h = getenv("HOME");
             if (h != NULL)
-                home = String(h);
+                home = h;
 
 #else
             char *h = getenv("HOMEPATH");
             char *d = getenv("HOMEDRIVE");
             if ((h != NULL)  && (d != NULL))
-                home = String(d) + h;
+                home = d + h;
 
 #endif  // __CYGWIN__
         }
@@ -338,19 +394,19 @@ For more information visit " DESC_MANUFACTURER_URL "\n\n");
 
         char *pref = getenv("MEDIATOMB_DATADIR");
         if (pref != NULL)
-            prefix = String(pref);
+            prefix = pref;
 
         if (!string_ok(prefix))
             prefix = _(PACKAGE_DATADIR);
 
         char *mgc = getenv("MEDIATOMB_MAGIC_FILE");
         if (mgc != NULL)
-            magic = String(mgc);
+            magic = mgc;
 
         if (!string_ok(magic))
             magic = nil;
 
-        ConfigManager::setStaticArgs(config_file, home, confdir, prefix, magic);
+        ConfigManager::setStaticArgs(config_file, home, confdir, prefix, magic, debug_logging);
         ConfigManager::getInstance();
     }
     catch (mxml::ParseException pe)
@@ -421,34 +477,42 @@ For more information visit " DESC_MANUFACTURER_URL "\n\n");
         if (devnull == -1)
         {
             log_error("Failed to open /dev/null: %s\n", strerror(errno));
+            exit(EXIT_FAILURE);
         }
-        dup(devnull);
-        dup(devnull);
+
+        redirect1 = dup(devnull);
+        if (redirect1 == -1)
+        {
+            log_error("Failed to redirect output: %s\n", strerror(errno));
+            close(devnull);
+            exit(EXIT_FAILURE);
+        }
+
+        redirect2 = dup(devnull);
+        if (redirect2 == -1)
+        {
+            log_error("Failed to redirect output: %s\n", strerror(errno));
+            close(devnull);
+            close(redirect1);
+            exit(EXIT_FAILURE);
+        }
+
     }
 
-    if (pid_file != nil)
+    if (pid_fd != NULL)
     {
         pid_t cur_pid;
-        FILE    *f;
         int     size;
 
         cur_pid = getpid();
         String pid = String::from(cur_pid);
 
-        f = fopen(pid_file.c_str(), "w");
-        if (f == NULL)
-        {                    
-            log_error("Could not write pid file %s : %s\n", pid_file.c_str(), strerror(errno));
-        }
-        else
-        {
-            size = fwrite(pid.c_str(), sizeof(char), pid.length(), f);
-            fclose(f);
-
-            if (size < pid.length())
-                log_error("Error when writing pid file %s : %s\n", pid_file.c_str(), strerror(errno));
-        }
+        size = fwrite(pid.c_str(), sizeof(char), pid.length(), pid_fd);
+        fclose(pid_fd);
 
+        if (size < pid.length())
+            log_error("Error when writing pid file %s : %s\n",
+                      pid_file.c_str(), strerror(errno));
     }
 
     main_thread_id = pthread_self();
@@ -518,7 +582,11 @@ For more information visit " DESC_MANUFACTURER_URL "\n\n");
             e.printStackTrace();
         }
         if (daemon)
+        {
             close(devnull);
+            close(redirect1);
+            close(redirect2);
+        }
         exit(EXIT_FAILURE);
     }
     catch (Exception e)
@@ -546,7 +614,11 @@ For more information visit " DESC_MANUFACTURER_URL "\n\n");
             {
                 e.printStackTrace();
                 if (daemon)
+                {
                     close(devnull);
+                    close(redirect1);
+                    close(redirect2);
+                }
                 exit(EXIT_FAILURE);
             }
         }
@@ -601,7 +673,11 @@ For more information visit " DESC_MANUFACTURER_URL "\n\n");
                               e.getMessage().c_str());
                     e.printStackTrace();
                     if (daemon)
+                    {
                         close(devnull);
+                        close(redirect1);
+                        close(redirect2);
+                    }
                     exit(EXIT_FAILURE);
                 }
                 
@@ -643,7 +719,11 @@ For more information visit " DESC_MANUFACTURER_URL "\n\n");
     log_close();
 
     if (daemon)
+    {
         close(devnull);
+        close(redirect1);
+        close(redirect2);
+    }
 
     exit(ret);
 }
