@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: mxml.h 1710 2008-03-01 15:56:36Z lww $
+    $Id: mxml.h 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file mxml.h
@@ -34,13 +34,26 @@
 
 #include "zmmf/zmmf.h"
 
+namespace mxml
+{
+enum mxml_value_type
+{
+    mxml_string_type,
+    mxml_bool_type,
+    mxml_null_type,
+    mxml_int_type
+};
+}
+
 #include "attribute.h"
 #include "context.h"
 #include "node.h"
 #include "element.h"
+#include "document.h"
 #include "xml_text.h"
 #include "comment.h"
 #include "parseexception.h"
 #include "parser.h"
+#include "xml_to_json.h"
 
 #endif // __MXML_H__
