@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: script.cc 1698 2008-02-23 20:48:30Z lww $
+    $Id: script.cc 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file script.cc
@@ -49,6 +49,18 @@
     #include "youtube_content_handler.h"
 #endif
 
+#ifdef WEBORAMA
+    #include "weborama_content_handler.h"
+#endif
+
+#ifdef ATRAILERS
+    #include "atrailers_content_handler.h"
+#endif
+
+#ifdef HAVE_LIBDVDNAV
+    #include "metadata/dvd_handler.h"
+#endif
+
 using namespace zmm;
 
 /*
@@ -75,7 +87,7 @@ String Script::getProperty(JSObject *obj, String name)
     str = JS_ValueToString(cx, val);
     if (! str)
         return nil;
-    return String(JS_GetStringBytes(str));
+    return JS_GetStringBytes(str);
 }
 
 int Script::getBoolProperty(JSObject *obj, String name)
@@ -292,50 +304,81 @@ Script::Script(Ref<Runtime> runtime) : Object()
 #ifdef YOUTUBE
     setIntProperty(glob, _("ONLINE_SERVICE_YOUTUBE"), (int)OS_YouTube);
 
-    setProperty(glob, _("YOUTUBE_AUXDATA_TAGS"), 
-            _(YOUTUBE_AUXDATA_TAGS));
+    setProperty(glob, _("YOUTUBE_AUXDATA_KEYWORDS"), 
+            _(YOUTUBE_AUXDATA_KEYWORDS));
     setProperty(glob, _("YOUTUBE_AUXDATA_AVG_RATING"), 
             _(YOUTUBE_AUXDATA_AVG_RATING));
     setProperty(glob, _("YOUTUBE_AUXDATA_AUTHOR"), 
             _(YOUTUBE_AUXDATA_AUTHOR));
-    setProperty(glob, _("YOUTUBE_AUXDATA_COMMENT_COUNT"), 
-            _(YOUTUBE_AUXDATA_COMMENT_COUNT));
+    setProperty(glob, _("YOUTUBE_AUXDATA_FEED"), 
+            _(YOUTUBE_AUXDATA_FEED));
     setProperty(glob, _("YOUTUBE_AUXDATA_VIEW_COUNT"), 
             _(YOUTUBE_AUXDATA_VIEW_COUNT));
+    setProperty(glob, _("YOUTUBE_AUXDATA_FAVORITE_COUNT"), 
+            _(YOUTUBE_AUXDATA_FAVORITE_COUNT));
     setProperty(glob, _("YOUTUBE_AUXDATA_RATING_COUNT"), 
             _(YOUTUBE_AUXDATA_RATING_COUNT));
-    setProperty(glob, _("YOUTUBE_AUXDATA_REQUEST"), 
-            _(YOUTUBE_AUXDATA_REQUEST));
     setProperty(glob, _("YOUTUBE_AUXDATA_CATEGORY"), 
             _(YOUTUBE_AUXDATA_CATEGORY));
-    setProperty(glob, _("YOUTUBE_AUXDATA_REQUEST_SUBNAME"), 
-            _(YOUTUBE_AUXDATA_REQUEST_SUBNAME));
+    setProperty(glob, _("YOUTUBE_AUXDATA_SUBREQUEST_NAME"), 
+            _(YOUTUBE_AUXDATA_SUBREQUEST_NAME));
+    setProperty(glob, _("YOUTUBE_AUXDATA_REQUEST"), 
+            _(YOUTUBE_AUXDATA_REQUEST));
+    setProperty(glob, _("YOUTUBE_AUXDATA_REGION"),
+            _(YOUTUBE_AUXDATA_REGION));
+
+    setIntProperty(glob, _("YOUTUBE_REQUEST_NONE"), (int)YT_request_none);
+    setIntProperty(glob, _("YOUTUBE_REQUEST_VIDEO_SEARCH"), 
+                  (int)YT_request_video_search);
+    setIntProperty(glob, _("YOUTUBE_REQUEST_STANDARD_FEED"), 
+                  (int)YT_request_stdfeed);
+    setIntProperty(glob, _("YOUTUBE_REQUEST_USER_FAVORITES"), 
+                  (int)YT_request_user_favorites);
+    setIntProperty(glob, _("YOUTUBE_REQUEST_USER_PLAYLISTS"), 
+                  (int)YT_request_user_playlists);
+    setIntProperty(glob, _("YOUTUBE_REQUEST_USER_SUBSCRIPTIONS"), 
+                   (int)YT_request_user_subscriptions);
+    setIntProperty(glob, _("YOUTUBE_REQUEST_USER_UPLOADS"), 
+                   (int)YT_request_user_uploads);
+#else
+    setIntProperty(glob, _("ONLINE_SERVICE_YOUTUBE"), -1);
+#endif//YOUTUBE
+
+#ifdef WEBORAMA
+    setIntProperty(glob, _("ONLINE_SERVICE_WEBORAMA"), (int)OS_Weborama);
+    setProperty(glob, _("WEBORAMA_AUXDATA_REQUEST_NAME"),
+                      _(WEBORAMA_AUXDATA_REQUEST_NAME));
+#else
+    setIntProperty(glob, _("ONLINE_SERVICE_WEBORAMA"), -1);
+#endif//WEBORAMAa
+
+#ifdef ATRAILERS
+    setIntProperty(glob, _("ONLINE_SERVICE_APPLE_TRAILERS"), (int)OS_ATrailers);
+    setProperty(glob, _("APPLE_TRAILERS_AUXDATA_POST_DATE"),
+                      _(ATRAILERS_AUXDATA_POST_DATE));
+#else
+    setIntProperty(glob, _("ONLINE_SERVICE_APPLE_TRAILERS"), -1);
+#endif//ATRAILERS
 
-    setIntProperty(glob, _("YOUTUBE_REQUEST_NONE"), (int)YT_list_none);
-    setIntProperty(glob, _("YOUTUBE_REQUEST_FAVORITE"), (int)YT_list_favorite);
-    setIntProperty(glob, _("YOUTUBE_REQUEST_BY_TAG"), (int)YT_list_by_tag);
-    setIntProperty(glob, _("YOUTUBE_REQUEST_BY_USER"), (int)YT_list_by_user);
-    setIntProperty(glob, _("YOUTUBE_REQUEST_FEATURED"), (int)YT_list_featured);
-    setIntProperty(glob, _("YOUTUBE_REQUEST_BY_PLAYLIST"), 
-                   (int)YT_list_by_playlist);
-    setIntProperty(glob, _("YOUTUBE_REQUEST_POPULAR"), (int)YT_list_popular);
-    setIntProperty(glob, _("YOUTUBE_REQUEST_BY_CATEGORY_AND_TAG"), 
-                   (int)YT_list_by_category_and_tag);
-#endif
 #ifdef SOPCAST
     setIntProperty(glob, _("ONLINE_SERVICE_SOPCAST"), (int)OS_SopCast);
-#endif
 #else
+    setIntProperty(glob, _("ONLINE_SERVICE_SOPCAST"), -1);
+#endif//SOPCAST
+
+#else // ONLINE SERVICES
     setIntProperty(glob, _("ONLINE_SERVICE_NONE"), 0);
+    setIntProperty(glob, _("ONLINE_SERVICE_YOUTUBE"), -1);
+    setIntProperty(glob, _("ONLINE_SERVICE_WEBORAMA"), -1);
+    setIntProperty(glob, _("ONLINE_SERVICE_SOPCAST"), -1);
+    setIntProperty(glob, _("ONLINE_SERVICE_APPLE_TRAILERS"), -1);
 #endif//ONLINE_SERVICES
 
     for (int i = 0; i < M_MAX; i++)
     {
         setProperty(glob, _(MT_KEYS[i].sym), _(MT_KEYS[i].upnp));
     }
-
-    setProperty(glob, _("UPNP_CLASS_CONTAINER_MUSIC"), 
-            _(UPNP_DEFAULT_CLASS_MUSIC_CONT));
+ 
     setProperty(glob, _("UPNP_CLASS_CONTAINER_MUSIC_ALBUM"),
             _(UPNP_DEFAULT_CLASS_MUSIC_ALBUM));
     setProperty(glob, _("UPNP_CLASS_CONTAINER_MUSIC_ARTIST"),
@@ -347,6 +390,10 @@ Script::Script(Ref<Runtime> runtime) : Object()
     setProperty(glob, _("UPNP_CLASS_ITEM"), _(UPNP_DEFAULT_CLASS_ITEM));
     setProperty(glob, _("UPNP_CLASS_ITEM_MUSIC_TRACK"),
             _(UPNP_DEFAULT_CLASS_MUSIC_TRACK));
+    setProperty(glob, _("UPNP_CLASS_ITEM_VIDEO"),
+            _(UPNP_DEFAULT_CLASS_VIDEO_ITEM));
+    setProperty(glob, _("UPNP_CLASS_ITEM_IMAGE"), 
+            _(UPNP_DEFAULT_CLASS_IMAGE_ITEM));
     setProperty(glob, _("UPNP_CLASS_PLAYLIST_CONTAINER"),
             _(UPNP_DEFAULT_CLASS_PLAYLIST_CONTAINER));
 
@@ -511,7 +558,7 @@ JSScript *Script::_load(zmm::String scriptPath)
     }
     catch (Exception e)
     {
-        throw _Exception(String("Failed to convert import script:") + e.getMessage().c_str());
+        throw _Exception(_("Failed to convert import script:") + e.getMessage().c_str());
     }
 
     scr = JS_CompileScript(cx, glob, scriptText.c_str(), scriptText.length(),
@@ -550,13 +597,14 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
     int objectType;
     int b;
     int i;
-    Ref<StringConverter> p2i;
-    Ref<StringConverter> i2i = StringConverter::i2i();
+    Ref<StringConverter> sc;
 
     if (this->whoami() == S_PLAYLIST)
     {
-        p2i = StringConverter::p2i();
+        sc = StringConverter::p2i();
     }
+    else
+        sc = StringConverter::i2i();
 
     objectType = getIntProperty(js, _("objectType"), -1);
     if (objectType == -1)
@@ -582,16 +630,11 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
     i = getIntProperty(js, _("parentID"), INVALID_OBJECT_ID);
     if (i != INVALID_OBJECT_ID)
         obj->setParentID(i);
+
     val = getProperty(js, _("title"));
     if (val != nil)
     {
-        if (this->whoami() == S_PLAYLIST)
-            val = p2i->convert(val);
-        else
-            // user has to take care of conversion in the script
-            // this is important when location is being set or used
-            val = i2i->convert(val);
-
+        val = sc->convert(val);
         obj->setTitle(val);
     }
     else
@@ -603,11 +646,7 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
     val = getProperty(js, _("upnpclass"));
     if (val != nil)
     {
-        if (this->whoami() == S_PLAYLIST)
-            val = p2i->convert(val);
-        else
-            val = i2i->convert(val);
-
+        val = sc->convert(val);
         obj->setClass(val);
     }
     else
@@ -635,7 +674,7 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
                     int j = val.toInt();
                     if (j > 0)
                     {
-                        obj->setMetadata(String(MT_KEYS[i].upnp), val);
+                        obj->setMetadata(MT_KEYS[i].upnp, val);
                         RefCast(obj, CdsItem)->setTrackNumber(j);
                     }
                     else
@@ -643,17 +682,21 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
                 }
                 else
                 {
-                    if (this->whoami() == S_PLAYLIST)
-                        val = p2i->convert(val);
-                    else
-                        val = i2i->convert(val);
-
-                    obj->setMetadata(String(MT_KEYS[i].upnp), val);
+                    val = sc->convert(val);
+                    obj->setMetadata(MT_KEYS[i].upnp, val);
                 }
             }
         }
         JS_RemoveRoot(cx, &js_meta);
     }
+    
+    // stuff that has not been exported to js
+    if (pcd != nil)
+    {
+        obj->setFlags(pcd->getFlags());
+        obj->setResources(pcd->getResources());
+        obj->setAuxData(pcd->getAuxData());
+    }
 
     // CdsItem
     if (IS_CDS_ITEM(objectType))
@@ -667,11 +710,7 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
         val = getProperty(js, _("mimetype"));
         if (val != nil)
         {
-            if (this->whoami() == S_PLAYLIST)
-                val = p2i->convert(val);
-            else
-                val = i2i->convert(val);
-
+            val = sc->convert(val);
             item->setMimeType(val);
         }
         else
@@ -683,11 +722,7 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
         val = getProperty(js, _("serviceID"));
         if (val != nil)
         {
-            if (this->whoami() == S_PLAYLIST)
-                val = p2i->convert(val);
-            else
-                val = i2i->convert(val);
-
+            val = sc->convert(val);
             item->setServiceID(val);
         }
 
@@ -696,11 +731,7 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
         val = getProperty(js, _("description"));
         if (val != nil)
         {
-            if (this->whoami() == S_PLAYLIST)
-                val = p2i->convert(val);
-            else
-                val = i2i->convert(val);
-
+            val = sc->convert(val);
             item->setMetadata(MetadataHandler::getMetaFieldName(M_DESCRIPTION), val);
         }
         else
@@ -762,23 +793,22 @@ Ref<CdsObject> Script::jsObject2cdsObject(JSObject *js, zmm::Ref<CdsObject> pcd)
             val = getProperty(js, _("protocol"));
             if (val != nil)
             {
-                if (this->whoami() == S_PLAYLIST)
-                    val = p2i->convert(val);
-                else
-                    val = i2i->convert(val);
-
-
+                val = sc->convert(val);
                 protocolInfo = renderProtocolInfo(item->getMimeType(), val);
             }
             else
             {
-                protocolInfo = renderProtocolInfo(item->getMimeType(), _(MIMETYPE_DEFAULT));
+                protocolInfo = renderProtocolInfo(item->getMimeType(), _(PROTOCOL));
             }
-            Ref<CdsResource> resource(new CdsResource(CH_DEFAULT));
-            resource->addAttribute(MetadataHandler::getResAttrName(
-                        R_PROTOCOLINFO), protocolInfo);
 
-            item->addResource(resource);
+            if (item->getResourceCount() == 0)
+            {
+                Ref<CdsResource> resource(new CdsResource(CH_DEFAULT));
+                resource->addAttribute(MetadataHandler::getResAttrName(
+                            R_PROTOCOLINFO), protocolInfo);
+
+                item->addResource(resource);
+            }
         }
     }
 
@@ -829,6 +859,7 @@ void Script::cdsObject2jsObject(Ref<CdsObject> obj, JSObject *js)
     if (val != nil)
         setProperty(js, _("location"), val);
 
+
     // TODO: boolean type
     i = obj->isRestricted();
     setIntProperty(js, _("restricted"), i);
@@ -860,6 +891,9 @@ void Script::cdsObject2jsObject(Ref<CdsObject> obj, JSObject *js)
             Ref<DictionaryElement> el = elements->get(i);
             setProperty(meta_js, el->getKey(), el->getValue());
         }
+
+        if (RefCast(obj, CdsItem)->getTrackNumber() > 0)
+            setProperty(meta_js, MetadataHandler::getMetaFieldName(M_TRACKNUMBER), String::from(RefCast(obj, CdsItem)->getTrackNumber())); 
     }
 
     // setting auxdata
@@ -867,35 +901,143 @@ void Script::cdsObject2jsObject(Ref<CdsObject> obj, JSObject *js)
         JSObject *aux_js = JS_NewObject(cx, NULL, NULL, js);
         setObjectProperty(js, _("aux"), aux_js);
         Ref<Dictionary> aux = obj->getAuxData();
-#ifdef YOUTUBE
-        int yt_request = (int)YT_list_none;
-        // put in meaningful names for YouTube specific enum values
-        String tmp = aux->get(_(YOUTUBE_AUXDATA_REQUEST));
-        if (string_ok(tmp))
-        {
-            yt_methods_t m = (yt_methods_t)tmp.toInt();
-            tmp = YouTubeService::getRequestName(m);
-            if (string_ok(tmp))
-            {
-                yt_request = (int)m;
-                aux->put(_(YOUTUBE_AUXDATA_REQUEST), tmp);
 
-                if (m == YT_list_by_category_and_tag)
+#ifdef HAVE_LIBDVDNAV
+        if (obj->getFlag(OBJECT_FLAG_DVD_IMAGE))
+        {
+            JSObject *aux_dvd = JS_NewObject(cx, NULL, NULL, js);
+            setObjectProperty(aux_js, _("DVD"), aux_dvd);
+
+            int title_count = obj->getAuxData(
+                                DVDHandler::renderKey(DVD_TitleCount)).toInt();
+
+            JSObject *titles = JS_NewArrayObject(cx, 0, NULL);
+            setObjectProperty(aux_dvd, _("titles"), titles);
+
+            for (int t = 0; t < title_count; t++)
+            {
+                JSObject *title = JS_NewObject(cx, NULL, NULL, js);
+                jsval val = OBJECT_TO_JSVAL(title);
+                JS_SetElement(cx, titles, t, &val);
+
+                setProperty(title, _("duration"), 
+                        obj->getAuxData(DVDHandler::renderKey(DVD_TitleDuration,
+                                        t)));
+
+                JSObject *audio_tracks = JS_NewArrayObject(cx, 0, NULL);
+                setObjectProperty(title, _("audio_tracks"), audio_tracks);
+
+                int audio_track_count = obj->getAuxData(
+                        DVDHandler::renderKey(DVD_AudioTrackCount, t)).toInt();
+
+                for (int a = 0; a < audio_track_count; a++)
                 {
-                    tmp = aux->get(_(YOUTUBE_AUXDATA_REQUEST_SUBNAME));
-                    if (string_ok(tmp))
-                    {
-                        yt_categories_t c = (yt_categories_t)tmp.toInt();
-                        tmp = YouTubeService::getCategoryName(c);
-                        if (string_ok(tmp))
-                            aux->put(_(YOUTUBE_AUXDATA_REQUEST_SUBNAME), tmp);
-                    }
+                    JSObject *track = JS_NewObject(cx, NULL, NULL, js);
+                    jsval val = OBJECT_TO_JSVAL(track);
+                    JS_SetElement(cx, audio_tracks, a, &val);
+
+                    setProperty(track, _("language"), obj->getAuxData(
+                                DVDHandler::renderKey(DVD_AudioTrackLanguage,
+                                    t, 0, a)));
+
+                    setProperty(track, _("format"), obj->getAuxData(
+                                DVDHandler::renderKey(DVD_AudioTrackFormat, 
+                                    t, 0, a)));
+                }
+
+                JSObject *chapters = JS_NewArrayObject(cx, 0, NULL);
+                setObjectProperty(title, _("chapters"), chapters);
+
+                int chapter_count = obj->getAuxData(DVDHandler::renderKey(DVD_ChapterCount, t)).toInt();
+
+                for (int c = 0; c < chapter_count; c++)
+                {
+                    JSObject *chapter = JS_NewObject(cx, NULL, NULL, js);
+                    jsval val = OBJECT_TO_JSVAL(chapter);
+                    JS_SetElement(cx, chapters, c, &val);
+
+                    setProperty(chapter, _("duration"), obj->getAuxData(
+                                DVDHandler::renderKey(DVD_ChapterRestDuration,
+                                    t, c)));
                 }
             }
-
-            setIntProperty(js, _("yt_request"), yt_request);
         }
+
 #endif
+#ifdef YOUTUBE
+        // put in meaningful names for YouTube specific enum values
+        String tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_AVG_RATING));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_AVG_RATING), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_KEYWORDS));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_KEYWORDS), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_AUTHOR));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_AUTHOR), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_FAVORITE_COUNT));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_FAVORITE_COUNT), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_VIEW_COUNT));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_VIEW_COUNT), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_RATING_COUNT));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_RATING_COUNT), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_FEED));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_FEED), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_SUBREQUEST_NAME));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_SUBREQUEST_NAME), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_CATEGORY));
+        if (string_ok(tmp))
+            aux->put(_(YOUTUBE_AUXDATA_CATEGORY), tmp);
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_REQUEST));
+        if (string_ok(tmp))
+        {
+            yt_requests_t req = (yt_requests_t)tmp.toInt();
+
+            // since subrequests do not actually produce any items they
+            // should not be visible to js
+            if (req == YT_subrequest_playlists)
+                req = YT_request_user_playlists;
+            else if (req == YT_subrequest_subscriptions)
+                req = YT_request_user_subscriptions;
+
+            setIntProperty(js, _("yt_request"), (int)req);
+            tmp = YouTubeService::getRequestName(req);
+            if (string_ok(tmp))
+                aux->put(_(YOUTUBE_AUXDATA_REQUEST), tmp);
+        }
+
+        tmp = obj->getAuxData(_(YOUTUBE_AUXDATA_REGION));
+        if (string_ok(tmp))
+        {
+            yt_regions_t reg = (yt_regions_t)tmp.toInt();
+            if (reg != YT_region_none)
+            {
+                tmp = YouTubeService::getRegionName(reg);
+                if (string_ok(tmp))
+                    aux->put(_(YOUTUBE_AUXDATA_REGION), tmp);
+            }
+        }
+#endif // YouTube
+#ifdef HAVE_ATRAILERSSSS
+        tmp = obj->getAuxData(_(ATRAILERS_AUXDATA_POST_DATE));
+        if (string_ok(tmp))
+            aux->put(_(ATRAILERS_AUXDATA_POST_DATE), tmp);
+#endif
+
         Ref<Array<DictionaryElement> > elements = aux->getElements();
         int len = elements->size();
         for (int i = 0; i < len; i++)
@@ -964,4 +1106,9 @@ String Script::convertToCharset(String str, charset_convert_t chr)
     return nil;
 }
 
+Ref<CdsObject> Script::getProcessedObject()
+{
+    return processed;
+}
+
 #endif // HAVE_JS
