@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: sql_storage.cc 1698 2008-02-23 20:48:30Z lww $
+    $Id: sql_storage.cc 2085 2010-03-25 15:27:12Z jin_eld $
 */
 
 /// \file sql_storage.cc
@@ -38,13 +38,11 @@
 #include "tools.h"
 #include "update_manager.h"
 #include "string_converter.h"
+#include "config_manager.h"
+#include "filesystem.h"
 
 using namespace zmm;
 
-#define LOC_DIR_PREFIX      'D'
-#define LOC_FILE_PREFIX     'F'
-#define LOC_VIRT_PREFIX     'V'
-#define LOC_ILLEGAL_PREFIX  'X'
 #define MAX_REMOVE_SIZE     10000
 #define MAX_REMOVE_RECURSION 500
 
@@ -132,6 +130,7 @@ SQLStorage::SQLStorage() : Storage()
 {
     table_quote_begin = '\0';
     table_quote_end = '\0';
+    lastID = INVALID_OBJECT_ID;
 }
 
 void SQLStorage::init()
@@ -142,6 +141,23 @@ void SQLStorage::init()
     Ref<StringBuffer> buf(new StringBuffer());
     *buf << SQL_QUERY_FOR_STRINGBUFFER;
     this->sql_query = buf->toString();
+   
+    if (ConfigManager::getInstance()->getBoolOption(CFG_SERVER_STORAGE_CACHING_ENABLED))
+    {
+        cache = Ref<StorageCache>(new StorageCache());
+        insertBufferOn = true;
+    }
+    else
+    {
+        cache = nil;
+        insertBufferOn = false;
+    }
+    
+    insertBufferEmpty = true;
+    insertBufferMutex = Ref<Mutex>(new Mutex());
+    insertBufferStatementCount = 0;
+    insertBufferByteCount = 0;
+    
     //log_debug("using SQL: %s\n", this->sql_query.c_str());
     
     //objectTitleCache = Ref<DSOHash<CdsObject> >(new DSOHash<CdsObject>(OBJECT_CACHE_CAPACITY));
@@ -166,6 +182,18 @@ void SQLStorage::init()
 */
 }
 
+void SQLStorage::dbReady()
+{
+    nextIDMutex = Ref<Mutex>(new Mutex());;
+    loadLastID();
+}
+
+void SQLStorage::shutdown()
+{
+    flushInsertBuffer();
+    shutdownDriver();
+}
+
 /*
 SQLStorage::~SQLStorage()
 {
@@ -190,9 +218,17 @@ Ref<CdsObject> SQLStorage::checkRefID(Ref<CdsObject> obj)
         }
         catch (Exception e)
         {
+            // this should never happen - but fail softly if compiled without debugging
+            assert(0);
+            throw _Exception(_("illegal refID was set"));
         }
     }
     
+    // this should never happen - but fail softly if compiled without debugging
+    // we do this assert to find code while debugging, that doesn't set the
+    // refID correctly
+    assert(0);
+    
     return findObjectByPath(location);
 }
 
@@ -228,6 +264,11 @@ Ref<Array<SQLStorage::AddUpdateTable> > SQLStorage::_addUpdateObject(Ref<CdsObje
             if (refObj == nil)
                 throw _Exception(_("OBJECT_FLAG_ONLINE_SERVICE and refID set but refID doesn't point to an existing object"));
         }
+        else if (IS_CDS_CONTAINER(objectType))
+        {
+            // in this case it's a playlist-container. that's ok
+            // we don't need to do anything
+        }
         else
             throw _Exception(_("refId set, but it makes no sense"));
     }
@@ -374,6 +415,7 @@ Ref<Array<SQLStorage::AddUpdateTable> > SQLStorage::_addUpdateObject(Ref<CdsObje
         cdsActiveItemSql->put(_("state"), quote(aitem->getState()));
     }
     
+    
     // check for a duplicate (virtual) object
     if (hasReference && ! isUpdate)
     {
@@ -417,6 +459,7 @@ void SQLStorage::addObject(Ref<CdsObject> obj, int *changedContainer)
         
         Ref<StringBuffer> fields(new StringBuffer(128));
         Ref<StringBuffer> values(new StringBuffer(128));
+        
         for (int j = 0; j < dataElements->size(); j++)
         {
             Ref<DictionaryElement> element = dataElements->get(j);
@@ -434,23 +477,54 @@ void SQLStorage::addObject(Ref<CdsObject> obj, int *changedContainer)
             *values << element->getValue();
         }
         
+        /* manually generate ID */
+        if (lastInsertID == INVALID_OBJECT_ID && tableName == _(CDS_OBJECT_TABLE))
+        {
+            lastInsertID = getNextID();
+            obj->setID(lastInsertID);
+            *fields << ',' << TQ("id");
+            *values << ',' << quote(lastInsertID);
+        }
+        /* -------------------- */
+        
         Ref<StringBuffer> qb(new StringBuffer(256));
         *qb << "INSERT INTO " << TQ(tableName) << " (" << fields->toString() <<
                 ") VALUES (" << values->toString() << ')';
                 
         log_debug("insert_query: %s\n", qb->toString().c_str());
         
-        if ( lastInsertID == INVALID_OBJECT_ID && tableName == _(CDS_OBJECT_TABLE))
+        /*
+        if (lastInsertID == INVALID_OBJECT_ID && tableName == _(CDS_OBJECT_TABLE))
         {
             lastInsertID = exec(qb, true);
             obj->setID(lastInsertID);
         }
-        else exec(qb);
+        else
+            exec(qb);
+        */
+        
+        if (! doInsertBuffering())
+            exec(qb);
+        else
+            addToInsertBuffer(qb);
     }
+    
+    /* add to cache */
+    if (cacheOn())
+    {
+        AUTOLOCK(cache->getMutex());
+        cache->addChild(obj->getParentID());
+        if (cache->flushed())
+            flushInsertBuffer();
+        addObjectToCache(obj, true);
+    }
+    /* ------------ */
 }
 
 void SQLStorage::updateObject(zmm::Ref<CdsObject> obj, int *changedContainer)
 {
+    flushInsertBuffer();
+    
     Ref<Array<AddUpdateTable> > data;
     if (obj->getID() == CDS_ID_FS_ROOT)
     {
@@ -495,10 +569,27 @@ void SQLStorage::updateObject(zmm::Ref<CdsObject> obj, int *changedContainer)
         
         exec(qb);
     }
+    /* add to cache */
+    addObjectToCache(obj);
+    /* ------------ */
 }
 
 Ref<CdsObject> SQLStorage::loadObject(int objectID)
 {
+    
+    /* check cache */
+    if (cacheOn())
+    {
+        AUTOLOCK(cache->getMutex());
+        Ref<CacheObject> cObj = cache->getObject(objectID);
+        if (cObj != nil)
+        {
+            if (cObj->knowsObject())
+                return cObj->getObject();
+        }
+    }
+    /* ----------- */
+    
 /*
     Ref<CdsObject> obj = objectIDCache->get(objectID);
     if (obj != nil)
@@ -522,6 +613,8 @@ Ref<CdsObject> SQLStorage::loadObject(int objectID)
 
 Ref<CdsObject> SQLStorage::loadObjectByServiceID(String serviceID)
 {
+    flushInsertBuffer();
+    
     Ref<StringBuffer> qb(new StringBuffer());
     *qb << SQL_QUERY << " WHERE " << TQD('f',"service_id") << '=' << quote(serviceID);
     Ref<SQLResult> res = select(qb);
@@ -536,6 +629,8 @@ Ref<CdsObject> SQLStorage::loadObjectByServiceID(String serviceID)
 
 Ref<IntArray> SQLStorage::getServiceObjectIDs(char servicePrefix)
 {
+    flushInsertBuffer();
+    
     Ref<IntArray> objectIDs(new IntArray());
     Ref<StringBuffer> qb(new StringBuffer());
     *qb << "SELECT " << TQ("id")
@@ -558,8 +653,10 @@ Ref<IntArray> SQLStorage::getServiceObjectIDs(char servicePrefix)
 
 Ref<Array<CdsObject> > SQLStorage::browse(Ref<BrowseParam> param)
 {
+    flushInsertBuffer();
+    
     int objectID;
-    int objectType;
+    int objectType = 0;
     
     bool getContainers = param->getFlag(BROWSE_CONTAINERS);
     bool getItems = param->getFlag(BROWSE_ITEMS);
@@ -569,22 +666,52 @@ Ref<Array<CdsObject> > SQLStorage::browse(Ref<BrowseParam> param)
     Ref<SQLResult> res;
     Ref<SQLRow> row;
     
-    Ref<StringBuffer> qb(new StringBuffer());
-    *qb << "SELECT " << TQ("object_type")
-        << " FROM " << TQ(CDS_OBJECT_TABLE)
-        << " WHERE " << TQ("id") << '=' << objectID;
-    res = select(qb);
-    if(res != nil && (row = res->nextRow()) != nil)
+    bool haveObjectType = false;
+    
+    /* check cache */
+    if (cacheOn())
     {
-        objectType = row->col(0).toInt();
+        AUTOLOCK(cache->getMutex());
+        Ref<CacheObject> cObj = cache->getObject(objectID);
+        if (cObj != nil && cObj->knowsObjectType())
+        {
+            objectType = cObj->getObjectType();
+            haveObjectType = true;
+        }
     }
-    else
+    /* ----------- */
+    
+    Ref<StringBuffer> qb(new StringBuffer());
+    if (! haveObjectType)
     {
-        throw _ObjectNotFoundException(_("Object not found: ") + objectID);
+        *qb << "SELECT " << TQ("object_type")
+            << " FROM " << TQ(CDS_OBJECT_TABLE)
+            << " WHERE " << TQ("id") << '=' << objectID;
+        res = select(qb);
+        if(res != nil && (row = res->nextRow()) != nil)
+        {
+            objectType = row->col(0).toInt();
+            haveObjectType = true;
+            
+            /* add to cache */
+            if (cacheOn())
+            {
+                AUTOLOCK(cache->getMutex());
+                cache->getObjectDefinitely(objectID)->setObjectType(objectType);
+                if (cache->flushed())
+                    flushInsertBuffer();
+            }
+            /* ------------ */
+        }
+        else
+        {
+            throw _ObjectNotFoundException(_("Object not found: ") + objectID);
+        }
+        
+        row = nil;
+        res = nil;
     }
     
-    row = nil;
-    res = nil;
     
     bool hideFsRoot = param->getFlag(BROWSE_HIDE_FS_ROOT);
     
@@ -597,9 +724,6 @@ Ref<Array<CdsObject> > SQLStorage::browse(Ref<BrowseParam> param)
         param->setTotalMatches(1);
     }
     
-    row = nil;
-    res = nil;
-    
     // order by code..
     qb->clear();
     if (param->getFlag(BROWSE_TRACK_SORT))
@@ -691,6 +815,23 @@ int SQLStorage::getChildCount(int contId, bool containers, bool items, bool hide
 {
     if (! containers && ! items)
         return 0;
+    
+    /* check cache */
+    if (cacheOn() && containers && items && ! (contId == CDS_ID_ROOT && hideFsRoot))
+    {
+        AUTOLOCK(cache->getMutex());
+        Ref<CacheObject> cObj = cache->getObject(contId);
+        if (cObj != nil)
+        {
+            if (cObj->knowsNumChildren())
+                return cObj->getNumChildren();
+            //cObj->debug();
+        }
+    }
+    /* ----------- */
+    
+    flushInsertBuffer();
+    
     Ref<SQLRow> row;
     Ref<SQLResult> res;
     Ref<StringBuffer> qb(new StringBuffer());
@@ -708,13 +849,27 @@ int SQLStorage::getChildCount(int contId, bool containers, bool items, bool hide
     res = select(qb);
     if (res != nil && (row = res->nextRow()) != nil)
     {
-        return row->col(0).toInt();
+        int childCount = row->col(0).toInt();
+        
+        /* add to cache */
+        if (cacheOn() && containers && items && ! (contId == CDS_ID_ROOT && hideFsRoot))
+        {
+            AUTOLOCK(cache->getMutex());
+            cache->getObjectDefinitely(contId)->setNumChildren(childCount);
+            if (cache->flushed())
+                flushInsertBuffer();
+        }
+        /* ------------ */
+        
+        return childCount;
     }
     return 0;
 }
 
 Ref<Array<StringBase> > SQLStorage::getMimeTypes()
 {
+    flushInsertBuffer();
+    
     Ref<Array<StringBase> > arr(new Array<StringBase>());
     
     Ref<StringBuffer> qb(new StringBuffer());
@@ -736,7 +891,7 @@ Ref<Array<StringBase> > SQLStorage::getMimeTypes()
     return arr;
 }
 
-Ref<SQLRow> SQLStorage::_findObjectByPath(String fullpath)
+Ref<CdsObject> SQLStorage::_findObjectByPath(String fullpath)
 {
     //log_debug("fullpath: %s\n", fullpath.c_str());
     fullpath = fullpath.reduce(DIR_SEPARATOR);
@@ -745,14 +900,36 @@ Ref<SQLRow> SQLStorage::_findObjectByPath(String fullpath)
     String path = pathAr->get(0);
     String filename = pathAr->get(1);
     
-    Ref<StringBuffer> qb(new StringBuffer());
     bool file = string_ok(filename);
     
     String dbLocation;
     if (file)
+    {
+        //flushInsertBuffer(); - not needed if correctly in cache (see below)
         dbLocation = addLocationPrefix(LOC_FILE_PREFIX, fullpath);
+    }
     else
         dbLocation = addLocationPrefix(LOC_DIR_PREFIX, path);
+    
+    
+    /* check cache */
+    if (cacheOn())
+    {
+        AUTOLOCK(cache->getMutex());
+        Ref<Array<CacheObject> > objects = cache->getObjects(dbLocation);
+        if (objects != nil)
+        {
+            for (int i = 0; i < objects->size(); i++)
+            {
+                Ref<CacheObject> cObj = objects->get(i);
+                if (cObj->knowsObject() && cObj->knowsVirtual() && !cObj->getVirtual())
+                    return cObj->getObject();
+            }
+        }
+    }
+    /* ----------- */
+    
+    Ref<StringBuffer> qb(new StringBuffer());
     *qb << SQL_QUERY
             << " WHERE " << TQD('f',"location_hash") << '=' << quote(stringHash(dbLocation))
             << " AND " << TQD('f',"location") << '=' << quote(dbLocation)
@@ -762,23 +939,25 @@ Ref<SQLRow> SQLStorage::_findObjectByPath(String fullpath)
     Ref<SQLResult> res = select(qb);
     if (res == nil)
         throw _Exception(_("error while doing select: ") + qb->toString());
-    return res->nextRow();
-}
-
-Ref<CdsObject> SQLStorage::findObjectByPath(String fullpath)
-{
-    Ref<SQLRow> row = _findObjectByPath(fullpath);
+    
+    
+    Ref<SQLRow> row = res->nextRow();
     if (row == nil)
         return nil;
     return createObjectFromRow(row);
 }
 
+Ref<CdsObject> SQLStorage::findObjectByPath(String fullpath)
+{
+    return _findObjectByPath(fullpath);
+}
+
 int SQLStorage::findObjectIDByPath(String fullpath)
 {
-    Ref<SQLRow> row = _findObjectByPath(fullpath);
-    if (row == nil)
+    Ref<CdsObject> obj = _findObjectByPath(fullpath);
+    if (obj == nil)
         return INVALID_OBJECT_ID;
-    return row->col(_id).toInt();
+    return obj->getID();
 }
 
 int SQLStorage::ensurePathExistence(String path, int *changedContainer)
@@ -820,10 +999,14 @@ int SQLStorage::createContainer(int parentID, String name, String path, bool isV
             throw _Exception(_("tried to create container with refID set, but refID doesn't point to an existing object"));
     }
     String dbLocation = addLocationPrefix((isVirtual ? LOC_VIRT_PREFIX : LOC_DIR_PREFIX), path);
+    
+    int newID = getNextID();
+    
     Ref<StringBuffer> qb(new StringBuffer());
     *qb << "INSERT INTO " 
         << TQ(CDS_OBJECT_TABLE)
         << " ("
+        << TQ("id") << ','
         << TQ("parent_id") << ','
         << TQ("object_type") << ','
         << TQ("upnp_class") << ','
@@ -831,6 +1014,7 @@ int SQLStorage::createContainer(int parentID, String name, String path, bool isV
         << TQ("location") << ','
         << TQ("location_hash") << ','
         << TQ("ref_id") << ") VALUES ("
+        << newID << ','
         << parentID << ','
         << OBJECT_TYPE_CONTAINER << ','
         << (string_ok(upnpClass) ? quote(upnpClass) : quote(_(UPNP_DEFAULT_CLASS_CONTAINER))) << ',' 
@@ -840,7 +1024,28 @@ int SQLStorage::createContainer(int parentID, String name, String path, bool isV
         << (refID > 0 ? quote(refID) : _(SQL_NULL))
         << ')';
         
-    return exec(qb, true);
+    exec(qb);
+    
+    /* inform cache */
+    if (cacheOn())
+    {
+        AUTOLOCK(cache->getMutex());
+        cache->addChild(parentID);
+        if (cache->flushed())
+            flushInsertBuffer();
+        Ref<CacheObject> cObj = cache->getObjectDefinitely(newID);
+        if (cache->flushed())
+            flushInsertBuffer();
+        cObj->setParentID(parentID);
+        cObj->setNumChildren(0);
+        cObj->setObjectType(OBJECT_TYPE_CONTAINER);
+        cObj->setLocation(path);
+    }
+    /* ------------ */
+    
+    return newID;
+    
+    //return exec(qb, true);
     
 }
 
@@ -961,7 +1166,7 @@ Ref<CdsObject> SQLStorage::createObjectFromRow(Ref<SQLRow> row)
     }
     
     if ( (obj->getRefID() && IS_CDS_PURE_ITEM(objectType)) ||
-        IS_CDS_ITEM(objectType) && ! IS_CDS_PURE_ITEM(objectType) )
+        (IS_CDS_ITEM(objectType) && ! IS_CDS_PURE_ITEM(objectType)) )
         obj->setVirtual(true);
     else
         obj->setVirtual(false); // gets set to true for virtual containers below
@@ -1044,11 +1249,15 @@ Ref<CdsObject> SQLStorage::createObjectFromRow(Ref<SQLRow> row)
     {
         throw _StorageException(nil, _("unknown object type: ")+ objectType);
     }
+    
+    addObjectToCache(obj);
     return obj;
 }
 
 int SQLStorage::getTotalFiles()
 {
+    flushInsertBuffer();
+    
     Ref<StringBuffer> query(new StringBuffer());
     *query << "SELECT COUNT(*) FROM " << TQ(CDS_OBJECT_TABLE) << " WHERE "
            << TQ("object_type") << " != " << quote(OBJECT_TYPE_CONTAINER);
@@ -1133,6 +1342,8 @@ Ref<Array<CdsObject> > SQLStorage::selectObjects(Ref<SelectParam> param)
 
 Ref<DBRHash<int> > SQLStorage::getObjects(int parentID, bool withoutContainer)
 {
+    flushInsertBuffer();
+    
     Ref<StringBuffer> q(new StringBuffer());
     *q << "SELECT " << TQ("id") << " FROM " << TQ(CDS_OBJECT_TABLE) << " WHERE ";
     if (withoutContainer)
@@ -1161,6 +1372,8 @@ Ref<DBRHash<int> > SQLStorage::getObjects(int parentID, bool withoutContainer)
 
 Ref<Storage::ChangedContainers> SQLStorage::removeObjects(zmm::Ref<DBRHash<int> > list, bool all)
 {
+    flushInsertBuffer();
+    
     hash_data_array_t<int> hash_data_array;
     list->getAll(&hash_data_array);
     int count = hash_data_array.size;
@@ -1266,6 +1479,8 @@ void SQLStorage::_removeObjects(Ref<StringBuffer> objectIDs, int offset)
 
 Ref<Storage::ChangedContainers> SQLStorage::removeObject(int objectID, bool all)
 {
+    flushInsertBuffer();
+    
     Ref<StringBuffer> q(new StringBuffer());
     *q << "SELECT " << TQ("object_type") << ',' << TQ("ref_id")
         << " FROM " << TQ(CDS_OBJECT_TABLE)
@@ -1567,6 +1782,12 @@ Ref<Storage::ChangedContainers> SQLStorage::_purgeEmptyContainers(Ref<ChangedCon
     }
     log_debug("end; changedContainers (upnp): %s\n", changedContainers->upnp->toCSV().c_str());
     log_debug("end; changedContainers (ui): %s\n", changedContainers->ui->toCSV().c_str());
+    
+    /* clear cache (for now) */
+    if (cacheOn())
+        cache->clear();
+    /* --------------------- */
+    
     return changedContainers;
 }
 
@@ -1746,7 +1967,11 @@ void SQLStorage::addAutoscanDirectory(Ref<AutoscanDirectory> adir)
         throw _Exception(_("addAutoscanDirectory called with adir==nil"));
     if (adir->getStorageID() >= 0)
         throw _Exception(_("tried to add autoscan directory with a storage id set"));
-    int objectID = findObjectIDByPath(adir->getLocation() + DIR_SEPARATOR);
+    int objectID;
+    if (adir->getLocation() == FS_ROOT_DIRECTORY)
+        objectID = CDS_ID_FS_ROOT;
+    else
+        objectID = findObjectIDByPath(adir->getLocation() + DIR_SEPARATOR);
     if (! adir->persistent() && objectID < 0)
         throw _Exception(_("tried to add non-persistent autoscan directory with an illegal objectID or location"));
     
@@ -2028,6 +2253,8 @@ Ref<IntArray> SQLStorage::_checkOverlappingAutoscans(Ref<AutoscanDirectory> adir
 
 Ref<IntArray> SQLStorage::getPathIDs(int objectID)
 {
+    flushInsertBuffer();
+    
     if (objectID == INVALID_OBJECT_ID)
         return nil;
     Ref<IntArray> pathIDs(new IntArray());
@@ -2070,3 +2297,94 @@ void SQLStorage::setFsRootName(String rootName)
         fsRootName = fsRootObj->getTitle();
     }
 }
+
+int SQLStorage::getNextID()
+{
+    if (lastID < CDS_ID_FS_ROOT)
+        throw _Exception(_("SQLStorage::getNextID() called, but lastID hasn't been loaded correctly yet"));
+    AUTOLOCK(nextIDMutex);
+    return ++lastID;
+}
+
+void SQLStorage::loadLastID()
+{
+    // we don't rely on automatic db generated ids, because of our caching
+    
+    Ref<StringBuffer> qb(new StringBuffer());
+    *qb << "SELECT MAX(" << TQ("id") << ')'
+        << " FROM " << TQ(CDS_OBJECT_TABLE);
+    Ref<SQLResult> res = select(qb);
+    if (res == nil)
+        throw _Exception(_("could not load lastID (res==nil)"));
+    
+    Ref<SQLRow> row = res->nextRow();
+    if (row == nil)
+        throw _Exception(_("could not load lastID (row==nil)"));
+    
+    lastID = row->col(0).toInt();
+    if (lastID < CDS_ID_FS_ROOT)
+        throw _Exception(_("could not load correct lastID (db not initialized?)"));
+}
+
+void SQLStorage::addObjectToCache(Ref<CdsObject> object, bool dontLock)
+{
+    if (cacheOn() && object != nil)
+    {
+        AUTOLOCK_DEFINE_ONLY();
+        if (! dontLock)
+            AUTOLOCK_NO_DEFINE(cache->getMutex());
+        Ref<CacheObject> cObj = cache->getObjectDefinitely(object->getID());
+        if (cache->flushed())
+            flushInsertBuffer();
+        cObj->setObject(object);
+        cache->checkLocation(cObj);
+    }
+}
+
+void SQLStorage::addToInsertBuffer(Ref<StringBuffer> query)
+{
+    assert(doInsertBuffering());
+    
+    _addToInsertBuffer(query);
+    
+    AUTOLOCK(mutex);
+    insertBufferEmpty = false;
+    insertBufferStatementCount++;
+    insertBufferByteCount += query->length();
+    
+    if (insertBufferByteCount > 102400)
+        flushInsertBuffer(true);
+}
+
+void SQLStorage::flushInsertBuffer(bool dontLock)
+{
+    if (! doInsertBuffering())
+        return;
+    //print_backtrace();
+    AUTOLOCK_DEFINE_ONLY();
+    if (! dontLock)
+        AUTOLOCK_NO_DEFINE(mutex);
+    if (insertBufferEmpty)
+        return;
+    _flushInsertBuffer();
+    log_debug("flushing insert buffer (%d statements)\n", insertBufferStatementCount);
+    insertBufferEmpty = true;
+    insertBufferStatementCount = 0;
+    insertBufferByteCount = 0;
+}
+
+void SQLStorage::clearFlagInDB(int flag)
+{
+    Ref<StringBuffer> qb(new StringBuffer(256));
+    *qb << "UPDATE "
+        << TQ(CDS_OBJECT_TABLE)
+        << " SET "
+        << TQ("flags")
+        << " = ("
+        << TQ("flags")
+        << "&~" << flag
+        << ") WHERE "
+        << TQ("flags")
+        << "&" << flag;
+    exec(qb);
+}
