@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: sqlite3_storage.cc 1698 2008-02-23 20:48:30Z lww $
+    $Id: sqlite3_storage.cc 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file sqlite3_storage.cc
@@ -69,6 +69,8 @@ Sqlite3Storage::Sqlite3Storage() : SQLStorage()
     startupError = nil;
     sqliteMutex = Ref<Mutex>(new Mutex());
     cond = Ref<Cond>(new Cond(sqliteMutex));
+    insertBuffer = nil;
+    dirty = false;
 }
 
 void Sqlite3Storage::init()
@@ -228,6 +230,8 @@ void Sqlite3Storage::init()
         this->addTask(RefCast(btask, SLTask));
         btask->waitForTask();
     }
+    
+    dbReady();
 }
 
 void Sqlite3Storage::_exec(const char *query)
@@ -239,7 +243,7 @@ String Sqlite3Storage::quote(String value)
 {
     char *q = sqlite3_mprintf("'%q'",
         (value == nil ? "" : value.c_str()));
-    String ret(q);
+    String ret = q;
     sqlite3_free(q);
     return ret;
 }
@@ -252,6 +256,8 @@ String Sqlite3Storage::getError(String query, String error, sqlite3 *db)
 
 Ref<SQLResult> Sqlite3Storage::select(const char *query, int length)
 {
+    //fprintf(stdout, "%s\n",query);
+    //fflush(stdout);
     Ref<SLSelectTask> ptask (new SLSelectTask(query));
     addTask(RefCast(ptask, SLTask));
     ptask->waitForTask();
@@ -260,6 +266,8 @@ Ref<SQLResult> Sqlite3Storage::select(const char *query, int length)
 
 int Sqlite3Storage::exec(const char *query, int length, bool getLastInsertId)
 {
+    //fprintf(stdout, "%s\n",query);
+    //fflush(stdout);
     Ref<SLExecTask> ptask (new SLExecTask(query, getLastInsertId));
     addTask(RefCast(ptask, SLTask));
     ptask->waitForTask();
@@ -307,6 +315,10 @@ void Sqlite3Storage::threadProc()
         try
         {
             task->run(&db, this);
+            if (task->didContamination())
+                dirty = true;
+            else if (task->didDecontamination())
+                dirty = false;
             task->sendSignal();
         }
         catch (Exception e)
@@ -325,7 +337,7 @@ void Sqlite3Storage::threadProc()
         sqlite3_close(db);
 }
 
-void Sqlite3Storage::addTask(zmm::Ref<SLTask> task)
+void Sqlite3Storage::addTask(zmm::Ref<SLTask> task, bool onlyIfDirty)
 {
     if (! taskQueueOpen)
         throw _Exception(_("sqlite3 task queue is already closed"));
@@ -334,11 +346,14 @@ void Sqlite3Storage::addTask(zmm::Ref<SLTask> task)
     {
         throw _Exception(_("sqlite3 task queue is already closed"));
     }
-    taskQueue->enqueue(task);
-    signal();
+    if (! onlyIfDirty || dirty)
+    {
+        taskQueue->enqueue(task);
+        signal();
+    }
 }
 
-void Sqlite3Storage::shutdown()
+void Sqlite3Storage::shutdownDriver()
 {
     log_debug("start\n");
     AUTOLOCK(sqliteMutex);
@@ -361,6 +376,27 @@ void Sqlite3Storage::storeInternalSetting(String key, String value)
     SQLStorage::exec(q);
 }
 
+void Sqlite3Storage::_addToInsertBuffer(Ref<StringBuffer> query)
+{
+    if (insertBuffer == nil)
+    {
+        insertBuffer = Ref<StringBuffer>(new StringBuffer());
+        *insertBuffer << "BEGIN TRANSACTION;";
+    }
+    
+    *insertBuffer << query << ';';
+}
+
+void Sqlite3Storage::_flushInsertBuffer()
+{
+    if (insertBuffer == nil)
+        return;
+    *insertBuffer << "COMMIT;";
+    SQLStorage::exec(insertBuffer);
+    insertBuffer->clear();
+    *insertBuffer << "BEGIN TRANSACTION;";
+}
+
 /* SLTask */
 
 SLTask::SLTask() : Object()
@@ -369,6 +405,8 @@ SLTask::SLTask() : Object()
     mutex = Ref<Mutex>(new Mutex());
     cond = Ref<Cond>(new Cond(mutex));
     error = nil;
+    contamination = false;
+    decontamination = false;
 }
 bool SLTask::is_running()
 {
@@ -432,7 +470,7 @@ void SLInitTask::run(sqlite3 **db, Sqlite3Storage *sl)
     char *err = NULL;
     ret = sqlite3_exec(
         *db,
-        (char *)buf,
+        (const char *)buf,
         NULL,
         NULL,
         &err
@@ -440,13 +478,14 @@ void SLInitTask::run(sqlite3 **db, Sqlite3Storage *sl)
     String error = nil;
     if (err != NULL)
     {
-        error = String(err);
+        error = err;
         sqlite3_free(err);
     }
     if(ret != SQLITE_OK)
     {
-        throw _StorageException(nil, sl->getError(String((char *)buf), error, *db));
+        throw _StorageException(nil, sl->getError((const char*)buf, error, *db));
     }
+    contamination = true;
 }
 
 #endif
@@ -475,12 +514,12 @@ void SLSelectTask::run(sqlite3 **db, Sqlite3Storage *sl)
     String error = nil;
     if (err != NULL)
     {
-        error = String(err);
+        error = err;
         sqlite3_free(err);
     }
     if(ret != SQLITE_OK)
     {
-        throw _StorageException(nil, sl->getError(String(query), error, *db));
+        throw _StorageException(nil, sl->getError(query, error, *db));
     }
     
     pres->row = pres->table;
@@ -509,15 +548,16 @@ void SLExecTask::run(sqlite3 **db, Sqlite3Storage *sl)
     String error = nil;
     if (err != NULL)
     {
-        error = String(err);
+        error = err;
         sqlite3_free(err);
     }
     if(res != SQLITE_OK)
     {
-        throw _StorageException(nil, sl->getError(String(query), error, *db));
+        throw _StorageException(nil, sl->getError(query, error, *db));
     }
     if (getLastInsertIdFlag)
         lastInsertId = sqlite3_last_insert_rowid(*db);
+    contamination = true;
 }
 
 /* SLBackupTask */
@@ -535,6 +575,8 @@ void SLBackupTask::run(sqlite3 **db, Sqlite3Storage *sl)
                 dbFilePath,
                 dbFilePath + ".backup"
             );
+            log_debug("sqlite3 backup successful\n");
+            decontamination = true;
         }
         catch (Exception e)
         {
@@ -615,7 +657,7 @@ void Sqlite3BackupTimerSubscriber::timerNotify(Ref<Object> sqlite3storage)
     Sqlite3Storage *storage = (Sqlite3Storage*)(sqlite3storage.getPtr());
     
     Ref<SLBackupTask> btask (new SLBackupTask(false));
-    storage->addTask(RefCast(btask, SLTask));
+    storage->addTask(RefCast(btask, SLTask), true);
 }
 
 #endif // HAVE_SQLITE3
