@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: tools.cc 1702 2008-02-23 22:00:57Z lww $
+    $Id: tools.cc 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file tools.cc
@@ -44,6 +44,7 @@
 #include <limits.h>
 #include <netdb.h>
 #include <string.h>
+#include "config_manager.h"
 
 #ifndef SOLARIS
     #include <net/if.h>
@@ -73,7 +74,7 @@ Ref<Array<StringBase> > split_string(String str, char sep, bool empty)
         char *pos = strchr(data, sep);
         if (pos == NULL)
         {
-            String part(data);
+            String part = data;
             ret->append(part);
             data = end;
         }
@@ -81,7 +82,7 @@ Ref<Array<StringBase> > split_string(String str, char sep, bool empty)
         {
             data++;
             if ((data < end) && empty)
-                ret->append(String(""));
+                ret->append(_(""));
         }
         else
         {
@@ -108,14 +109,14 @@ Ref<Array<StringBase> > split_path(String str)
     {
         /* there is only one separator at the beginning "/..." or "/" */
         ret->append(_(_DIR_SEPARATOR));
-        String filename(data+1);
+        String filename = data + 1;
         ret->append(filename);
     }
     else
     {
         String path(data, pos);
         ret->append(path);
-        String filename(data + pos + 1);
+        String filename = data + pos + 1;
         ret->append(filename);
     }
     return ret;
@@ -211,7 +212,7 @@ bool is_executable(String path, int *err)
 
 String find_in_path(String exec)
 {
-    String PATH = String(getenv("PATH"));
+    String PATH = getenv("PATH");
     if (!string_ok(PATH))
         return nil;
 
@@ -303,8 +304,8 @@ String hex_decode_string(String encoded)
     Ref<StringBuffer> buf(new StringBuffer(len / 2));
     for (int i = 0; i < len; i += 2)
     {
-        char *chi = strchr(HEX_CHARS, ptr[i]);
-        char *clo = strchr(HEX_CHARS, ptr[i + 1]);
+        const char *chi = strchr(HEX_CHARS, ptr[i]);
+        const char *clo = strchr(HEX_CHARS, ptr[i + 1]);
         int hi, lo;
         
         if (chi)
@@ -397,7 +398,7 @@ String url_unescape(String str)
             char clo = data[i++];
             int hi, lo;
 
-            char *pos;
+            const char *pos;
 
             pos = strchr(hex, chi);
             if (!pos)
@@ -459,7 +460,7 @@ String mt_strerror(int mt_errno)
     FREE(buffer);
     return errStr;
 #else
-    return String(strerror(errno));
+    return strerror(errno);
 #endif
 }
 
@@ -669,6 +670,22 @@ String secondsToHMS(int seconds)
     return String::take(str);
 }
 
+int HMSToSeconds(String time)
+{
+    if (!string_ok(time))
+    {
+        log_warning("Could not convert time representation to seconds!\n");
+        return 0;
+    }
+
+    int hours = 0;
+    int minutes = 0;
+    int seconds = 0;
+    sscanf(time.c_str(), "%d:%d:%d", &hours, &minutes, &seconds);
+
+    return (hours * 3600) + (minutes * 60) + seconds;
+}
+
 #ifdef HAVE_MAGIC
 String get_mime_type(magic_set *ms, Ref<RExp> reMimetype, String file)
 {
@@ -682,7 +699,7 @@ String get_mime_type(magic_set *ms, Ref<RExp> reMimetype, String file)
         return nil;
     }
 
-    String mime_type(mt);
+    String mime_type = mt;
 
     Ref<Matcher> matcher = reMimetype->matcher(mime_type, 2);
     if (matcher->next())
@@ -706,7 +723,7 @@ String get_mime_type_from_buffer(magic_set *ms, Ref<RExp> reMimetype,
         return nil;
     }
 
-    String mime_type(mt);
+    String mime_type = mt;
 
     Ref<Matcher> matcher = reMimetype->matcher(mime_type, 2);
     if (matcher->next())
@@ -752,8 +769,8 @@ bool check_resolution(String resolution, int *x, int *y)
     if (string_ok(parts->get(0)) && 
         string_ok(parts->get(1)))
         {
-            int _x = String(parts->get(0)->data).toInt();
-            int _y = String(parts->get(1)->data).toInt();
+            int _x = _(parts->get(0)->data).toInt();
+            int _y = _(parts->get(1)->data).toInt();
 
             if ((_x > 0) && (_y > 0))
             {
@@ -824,7 +841,7 @@ String escape(String string, char escape_char, char to_escape)
     *str = '\0';
     
     stringBase->len = strlen(stringBase->data);
-    return String(stringBase->data);
+    return stringBase->data;
 }
 
 String unescape(String string, char escape)
@@ -981,15 +998,13 @@ String intArrayToCSV(int *array, int size)
 
 void getTimespecNow(struct timespec *ts)
 {
-#ifdef HAVE_CLOCK_GETTIME 
-    clock_gettime(CLOCK_REALTIME, ts);
-#else
-    
     struct timeval tv;
-    gettimeofday(&tv, NULL);
+    int ret = gettimeofday(&tv, NULL);
+    if (ret != 0)
+        throw _Exception(_("gettimeofday failed: ") + mt_strerror(errno));
+    
     ts->tv_sec = tv.tv_sec;
     ts->tv_nsec = tv.tv_usec * 1000;
-#endif
 }
 
 long getDeltaMillis(struct timespec *first)
@@ -1054,14 +1069,7 @@ String normalizePath(String path)
 #ifndef __CYGWIN__
     if (path.charAt(0) != DIR_SEPARATOR)
 #else
-    #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
-    #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
-    #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
-    #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
-    #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
-    #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
-    #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
-    #warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
+    #error !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! this function is not finished for Cygwin
     for (int i = 0; i < 20; i++)
         print_backtrace();
     /// \todo this doesn't seem to be correct...
@@ -1113,7 +1121,7 @@ String normalizePath(String path)
         *(str++) = DIR_SEPARATOR;
     
     *str = 0;
-    
+    result->len = strlen(result->data);
     return String(result);
 }
 
@@ -1150,14 +1158,16 @@ String interfaceToIP(String interface)
     local_socket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
     if (local_socket < 0)
     {
-        log_error("Could not create local socket\n");
+        log_error("Could not create local socket: %s\n", 
+                  mt_strerror(errno).c_str());
         return nil;
     }
 
     iflist = iflist_free = if_nameindex();
     if (iflist == NULL)
     {
-        log_error("Could not get interface list\n");
+        log_error("Could not get interface list: %s\n", 
+                  mt_strerror(errno).c_str());
         close(local_socket);
         return nil;
     }
@@ -1169,14 +1179,15 @@ String interfaceToIP(String interface)
             strncpy(if_request.ifr_name, iflist->if_name, IF_NAMESIZE);
             if (ioctl(local_socket, SIOCGIFADDR, &if_request) != 0)
             {
-                log_error("Could not determine interface address\n");
+                log_error("Could not determine interface address: %s\n", 
+                          mt_strerror(errno).c_str());
                 close(local_socket);
                 if_freenameindex(iflist_free);
                 return nil;
             }
 
             memcpy(&local_address, &if_request.ifr_addr, sizeof(if_request.ifr_addr));
-            String ip = String(inet_ntoa(local_address.sin_addr));
+            String ip = inet_ntoa(local_address.sin_addr);
             if_freenameindex(iflist_free);
             close(local_socket);
             return ip;
@@ -1201,6 +1212,9 @@ bool validateYesNo(String value)
 Ref<Array<StringBase> > parseCommandLine(String line, String in, String out)
 {
     Ref<Array<StringBase> > params = split_string(line, ' ');
+    if ((in == nil) && (out == nil))
+        return params;
+
     for (int i = 0; i < params->size(); i++)
     {
         String param = params->get(i);
@@ -1302,6 +1316,12 @@ bool isTheora(String ogg_filename)
     FILE *f;
     char buffer[7];
     f = fopen(ogg_filename.c_str(), "rb");
+
+    if (!f)
+    {
+        throw _Exception(_("Error opening ") + ogg_filename + _(" : ") + 
+                    mt_strerror(errno));
+    }
     
     if (fread(buffer, 1, 4, f) != 4)
     {
@@ -1337,6 +1357,86 @@ bool isTheora(String ogg_filename)
     return true;
 }
 
+String get_last_path(String location)
+{
+    String path;
+
+    int last_slash = location.rindex(DIR_SEPARATOR);
+    if (last_slash > 0)
+    {
+        path = location.substring(0, last_slash);
+        int slash = path.rindex(DIR_SEPARATOR);
+        if ((slash >= 0) && ((slash + 1) < path.length()))
+            path = path.substring(slash+1);
+    }
+
+    return path;
+}
+
+
+ssize_t getValidUTF8CutPosition(zmm::String str, size_t cutpos)
+{
+    ssize_t pos = -1;
+    size_t len = str.length();
+
+    if ((len == 0) || (cutpos > len))
+        return pos;
+
+    printf("Character at cut position: %0x\n", (char)str.charAt(cutpos));
+    printf("Character at cut-1 position: %0x\n", (char)str.charAt(cutpos-1));
+    printf("Character at cut-2 position: %0x\n", (char)str.charAt(cutpos-2));
+    printf("Character at cut-3 position: %0x\n", (char)str.charAt(cutpos-3));
+
+    // > 0x7f, we are dealing with a non-ascii character
+    if (str.charAt(cutpos) & 0x80)
+    {
+        // check if we are at byte 2
+        if (((cutpos-1) >= 0) && 
+             (((str.charAt(cutpos-1) & 0xc2) == 0xc2) ||
+              ((str.charAt(cutpos-1) & 0xe2) == 0xe2) ||
+              ((str.charAt(cutpos-1) & 0xf0) == 0xf0)))
+            pos = cutpos - 1;
+        // check if we are at byte 3
+        else if (((cutpos - 2) >= 0) && 
+                (((str.charAt(cutpos-2) & 0xe2) == 0xe2) ||
+                 ((str.charAt(cutpos-2) & 0xf0) == 0xf0)))
+            pos = cutpos - 2;
+        // we must be at byte 4 then...
+        else if ((cutpos - 3) >= 0)
+            pos = cutpos - 3;
+    }
+    else
+        pos = cutpos;
+
+    return pos;
+}
+
+#ifdef EXTEND_PROTOCOLINFO
+String getDLNAtransferHeader(String mimeType, String header)
+{
+        if (ConfigManager::getInstance()->getBoolOption(CFG_SERVER_EXTEND_PROTOCOLINFO))
+        {
+            String transfer_parameter;
+            if (mimeType.startsWith(_("image")))
+                transfer_parameter = _(D_HTTP_TRANSFER_MODE_INTERACTIVE);
+            else if (mimeType.startsWith(_("audio")) ||
+                     mimeType.startsWith(_("video")))
+                transfer_parameter = _(D_HTTP_TRANSFER_MODE_STREAMING);
+
+            if (string_ok(transfer_parameter))
+            {
+                if (string_ok(header))
+                    header = header + _("\r\n");
+
+                header = header + D_HTTP_TRANSFER_MODE_HEADER +
+                         transfer_parameter;
+            }
+        }
+
+    return header;
+}
+#endif
+
 #ifndef HAVE_FFMPEG
 String getAVIFourCC(zmm::String avi_filename)
 {
@@ -1388,7 +1488,7 @@ String getAVIFourCC(zmm::String avi_filename)
 }
 #endif
 
-#ifdef LOG_TOMBDEBUG
+#ifdef TOMBDEBUG
 
 void profiling_thread_check(struct profiling_t *data)
 {
@@ -1439,13 +1539,12 @@ void profiling_end(struct profiling_t *data)
     }
     else
     {
-        sum->tv_nsec += 1000000000 + now.tv_nsec - last_start->tv_nsec;
-        //log_debug("adding 1 sec %ld nsec\n", 1000000000 + now.tv_nsec - last_start->tv_nsec);
-        sum->tv_sec ++;
+        sum->tv_nsec += 1000000000L - last_start->tv_nsec + now.tv_nsec;
+        sum->tv_sec --;
     }
-    if (sum->tv_nsec >= 1000000000)
+    if(sum->tv_nsec >= 1000000000L)
     {
-        sum->tv_nsec -= 1000000000;
+        sum->tv_nsec -= 1000000000L;
         sum->tv_sec ++;
     }
     
@@ -1466,3 +1565,62 @@ void profiling_print(struct profiling_t *data)
 }
 
 #endif
+
+#ifdef SOPCAST
+/// \brief 
+int find_local_port(unsigned short range_min, unsigned short range_max)
+{
+    int fd;
+    int retry_count = 0;
+    int port;
+    struct sockaddr_in server_addr;
+    struct hostent *server;
+
+    if (range_min > range_max)
+    {
+        log_error("min port range > max port range!\n");
+        return -1;
+    }
+
+    
+    do
+    {
+        port =  rand () % (range_max - range_min) + range_min;
+
+        fd = socket(AF_INET, SOCK_STREAM, 0);
+        if (fd < 0)
+        {
+            log_error("could not determine free port: "
+                      "error creating socket (%s)\n", mt_strerror(errno).c_str());
+            return -1;
+        }
+
+        server = gethostbyname("127.0.0.1");
+        if (server == NULL)
+        {
+            log_error("could not resolve localhost\n");
+            close(fd);
+            return -1;
+        }
+
+        memset(&server_addr, 0, sizeof(server_addr));
+        server_addr.sin_family = AF_INET;
+        memcpy(&server_addr.sin_addr.s_addr, server->h_addr, server->h_length);
+        server_addr.sin_port = htons(port);
+
+        if (connect(fd, (struct sockaddr *)&server_addr, 
+            sizeof(server_addr)) == -1)
+        {
+            close(fd);
+            return port;
+        }
+
+        retry_count++;
+
+    } while (retry_count < USHRT_MAX);
+
+    log_error("Could not find free port on localhost\n");
+
+    return -1;
+}
+#endif
