@@ -7,7 +7,7 @@
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
-    Copyright (C) 2006-2008 Gena Batyan <bgeradz@mediatomb.cc>,
+    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
     
@@ -24,7 +24,7 @@
     version 2 along with MediaTomb; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: web_update.cc 1698 2008-02-23 20:48:30Z lww $
+    $Id: web_update.cc 2081 2010-03-23 20:18:00Z lww $
 */
 
 /// \file web_update.cc
@@ -41,13 +41,7 @@
 using namespace zmm;
 using namespace mxml;
 
-/*
-web::autoscan::autoscan() : WebRequestHandler()
-{
-}
-*/
-
-void web::update::process()
+void web::voidType::process()
 {
     check_request();
 }
