@@ -32,7 +32,7 @@
     
     TombUPnP - a library for developing UPnP applications.
     
-    Copyright (C) 2006-2008 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
+    Copyright (C) 2006-2010 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -47,7 +47,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: ThreadPool.c 1698 2008-02-23 20:48:30Z lww $
+    $Id: ThreadPool.c 2081 2010-03-23 20:18:00Z lww $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -141,7 +141,7 @@ SetPolicyType( PolicyType in )
 static int
 SetPriority( ThreadPriority priority )
 {
-
+#if defined(HAVE_SCHED_GET_PRIORITY_MIN) && defined(HAVE_SCHED_GET_PRIORITY_MAX)
     int currentPolicy;
     int minPriority = 0;
     int maxPriority = 0;
@@ -151,6 +151,7 @@ SetPriority( ThreadPriority priority )
 
     pthread_getschedparam( ithread_self(  ), &currentPolicy,
                            &newPriority );
+
     minPriority = sched_get_priority_min( currentPolicy );
     maxPriority = sched_get_priority_max( currentPolicy );
     midPriority = ( maxPriority - minPriority ) / 2;
@@ -170,10 +171,11 @@ SetPriority( ThreadPriority priority )
     };
 
     newPriority.sched_priority = actPriority;
-
     return pthread_setschedparam( ithread_self(  ), currentPolicy,
                                   &newPriority );
-
+#else
+    return 0;
+#endif
 }
 
 /****************************************************************************
@@ -370,7 +372,7 @@ tp->stats.totalJobsLQ++; tp->stats.totalTimeLQ += diff; break; default:
     gettimeofday( &t, NULL );
 #if defined(WIN32)
     srand( ( unsigned int )(t.tv_usec/1000) + (unsigned int)ithread_get_current_thread_id(  ).p );
-#elif defined(__FreeBSD__) || defined (__APPLE__)
+#elif defined(__FreeBSD__) || defined (__APPLE__) || defined(__OpenBSD__) || defined(__NetBSD__)
     srand( ( unsigned int )(t.tv_usec/1000) + (unsigned int)ithread_get_current_thread_id(  ) );
 #else
     srand( ( unsigned int )(t.tv_usec/1000) + ithread_get_current_thread_id(  ) );
