@@ -32,7 +32,7 @@
     
     TombUPnP - a library for developing UPnP applications.
     
-    Copyright (C) 2006-2008 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
+    Copyright (C) 2006-2010 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -47,12 +47,16 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: webserver.c 1698 2008-02-23 20:48:30Z lww $
+    $Id: webserver.c 2081 2010-03-23 20:18:00Z lww $
 */
 /************************************************************************
 * Purpose: This file defines the Web Server and has functions to carry out 
 * operations of the Web Server.										
 ************************************************************************/
+#ifdef HAVE_CONFIG_H
+    #include "autoconfig.h"
+#endif
+
 
 #include "config.h"
 #include <assert.h>
@@ -629,6 +633,7 @@ get_file_info( IN const char *filename,
     int rc = 0;
 
     info->content_type = NULL;
+    info->force_chunked = FALSE;
 
     code = stat( filename, &s );
     if( code == -1 ) {
@@ -733,6 +738,7 @@ get_alias( IN const char *request_file,
         info->file_length = alias->doc.length;
         info->is_readable = TRUE;
         info->is_directory = FALSE;
+        info->force_chunked = FALSE;
         info->last_modified = alias->last_modified;
         info->http_header = NULL;
     }
@@ -1259,6 +1265,7 @@ process_request( IN http_message_t * req,
     request_doc = NULL;
     finfo.content_type = NULL;
     finfo.http_header = NULL;
+    finfo.force_chunked = FALSE;
     *Fp = NULL;
     //membuffer_init( &content_type );
     alias_grabbed = FALSE;
@@ -1445,6 +1452,8 @@ process_request( IN http_message_t * req,
     }
 
     RespInstr->ReadSendSize = finfo.file_length;
+    if (finfo.force_chunked == TRUE)
+        RespInstr->IsChunkActive = TRUE;
 
     //Check other header field.
     if( ( err_code =
