@@ -32,7 +32,7 @@
     
     TombUPnP - a library for developing UPnP applications.
     
-    Copyright (C) 2006-2008 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
+    Copyright (C) 2006-2010 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -47,7 +47,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: uri.c 1698 2008-02-23 20:48:30Z lww $
+    $Id: uri.c 2081 2010-03-23 20:18:00Z lww $
 */
 
 /************************************************************************
@@ -61,7 +61,7 @@
 #include "config.h"
 #include "uri.h"
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__NetBSD__)
     #ifdef HAVE_LWRES_NETDB_H
         #include <lwres/netdb.h>
     #endif
@@ -639,7 +639,7 @@ parse_hostport( const char *in,
         int errCode = 0;
 
         //call gethostbyname_r (reentrant form of gethostbyname)
-#if defined(WIN32) || defined(__APPLE__) || defined(__CYGWIN__)
+#if defined(WIN32) || defined(__APPLE__) || defined(__CYGWIN__) || defined(__OpenBSD__)
         h=gethostbyname(temp_host_name);
 #elif defined(SOLARIS)
         h = gethostbyname_r( temp_host_name,
