@@ -36,6 +36,10 @@
 * service 
 ************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+    #include "autoconfig.h"
+#endif
+
 #include "config.h"
 #include "service_table.h"
 
