@@ -32,7 +32,7 @@
     
     TombUPnP - a library for developing UPnP applications.
     
-    Copyright (C) 2006-2008 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
+    Copyright (C) 2006-2010 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -47,8 +47,12 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: soap_ctrlpt.c 1698 2008-02-23 20:48:30Z lww $
+    $Id: soap_ctrlpt.c 2095 2010-04-06 19:59:57Z jin_eld $
 */
+#ifdef HAVE_CONFIG_H
+    #include "autoconfig.h"
+#endif
+
 
 #include "config.h"
 #ifdef INCLUDE_CLIENT_APIS
@@ -674,7 +678,7 @@ SoapSendAction( IN char *action_url,
 
     // make request msg
     request.size_inc = 50;
-    if( http_MakeMessage( &request, 1, 1, "q" "N" "s" "sssbs" "U" "c" "bbb", 
+    if( http_MakeMessage( &request, 1, 1, "q" "N" "s" "sssbs" "UC" "c" "bbb", 
                           SOAPMETHOD_POST, 
                           &url, (off_t)(xml_start_len + action_str_len + xml_end_len),   // content-length
                           ContentTypeHeader,
@@ -832,7 +836,7 @@ SoapSendActionEx( IN char *action_url,
 
     // make request msg
     request.size_inc = 50;
-    if( http_MakeMessage( &request, 1, 1, "q" "N" "s" "sssbs" "U" "c" "bbbbbbb", 
+    if( http_MakeMessage( &request, 1, 1, "q" "N" "s" "sssbs" "UC" "c" "bbbbbbb", 
                           SOAPMETHOD_POST, 
                           &url, (off_t)(xml_start_len + 
                                         xml_header_start_len + 
@@ -945,7 +949,7 @@ SoapGetServiceVarStatus( IN char *action_url,
     }
     // make headers
     request.size_inc = 50;
-    if( http_MakeMessage( &request, 1, 1, "Q" "sbc" "N" "s" "s" "U" "c" "sss", 
+    if( http_MakeMessage( &request, 1, 1, "Q" "sbc" "N" "s" "s" "UC" "c" "sss", 
                           SOAPMETHOD_POST, 
                           path.buf, 
                           (size_t)path.length, 
