@@ -32,7 +32,7 @@
     
     TombUPnP - a library for developing UPnP applications.
     
-    Copyright (C) 2006-2008 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
+    Copyright (C) 2006-2010 Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
     
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -47,8 +47,12 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
     
-    $Id: ssdp_device.c 1698 2008-02-23 20:48:30Z lww $
+    $Id: ssdp_device.c 2081 2010-03-23 20:18:00Z lww $
 */
+#ifdef HAVE_CONFIG_H
+    #include "autoconfig.h"
+#endif
+
 
 #include "config.h"
 #ifdef INCLUDE_DEVICE_APIS
